@@ -1,4 +1,5 @@
-# Copyright 2012-2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2012-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,19 +14,21 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario@lassnig.net>, 2012-2020
-# - Vincent Garonne <vgaronne@gmail.com>, 2012-2017
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2020
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2017
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013-2014
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2017
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2014
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Gabriele <sucre.91@hotmail.it>, 2019
-# - elichad <eli.chadwick.256@gmail.com>, 2020
-# - patrick-austin <patrick.austin@stfc.ac.uk>, 2020
-#
-#  PY3K COMPATIBLE
+# - Gabriele Fronze' <gfronze@cern.ch>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
+# - Tomas Javurek <tomas.javurek@cern.ch>, 2020
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 from rucio.api import permission
 from rucio.common import exception
@@ -487,6 +490,23 @@ def get_distance(source, destination, issuer, vo='def'):
     return [api_update_return_dict(d) for d in distances]
 
 
+def delete_distance(source, destination, issuer, vo='def'):
+    """
+    Delete distances with the given RSE ids.
+
+    :param source: The source RSE.
+    :param destination: The destination RSE.
+    :param issuer: The issuer account.
+    :param vo: The VO to act on.
+    """
+    kwargs = {'source': source, 'destination': destination}
+    if not permission.has_permission(issuer=issuer, vo=vo, action='delete_distance', kwargs=kwargs):
+        raise exception.AccessDenied('Account %s can not update RSE distances' % issuer)
+
+    return distance_module.delete_distances(src_rse_id=rse_module.get_rse_id(source, vo=vo),
+                                            dest_rse_id=rse_module.get_rse_id(destination, vo=vo))
+
+
 def add_qos_policy(rse, qos_policy, issuer, vo='def'):
     """
     Add a QoS policy from an RSE.
