@@ -1,20 +1,27 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# -*- coding: utf-8 -*-
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
+# you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2013
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2013-2014
-# - Martin Barisits, <martin.barisits@cern.ch>, 2014
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2019
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2018
-# - Brandon White, <bjwhite@fnal.gov>, 2019
-# - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2015
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2014
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2021
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 
 from sqlalchemy.orm.exc import NoResultFound
 
@@ -170,7 +177,7 @@ def update_account_counter_history(account, rse_id, session=None):
     :param rse_id:   The rse_id to update.
     :param session:  Database session in use.
     """
-    AccountUsageHistory = models.AccountUsage.__history_mapper__.class_
+    AccountUsageHistory = models.AccountUsageHistory
     try:
         counter = session.query(models.AccountUsage).filter_by(rse_id=rse_id, account=account).one()
         AccountUsageHistory(rse_id=rse_id, account=account, files=counter.files, bytes=counter.bytes).save(session=session)
@@ -185,6 +192,6 @@ def fill_account_counter_history_table(session=None):
 
     :param session:  Database session in use.
     """
-    AccountUsageHistory = models.AccountUsage.__history_mapper__.class_
+    AccountUsageHistory = models.AccountUsageHistory
     for usage in session.query(models.AccountUsage).all():
         AccountUsageHistory(rse_id=usage['rse_id'], account=usage['account'], files=usage['files'], bytes=usage['bytes']).save(session=session)
