@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2020 CERN
+# Copyright 2012-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
 # - Martin Barisits <martin.barisits@cern.ch>, 2012-2019
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2017
 # - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
@@ -24,9 +24,8 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
 # - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
 
 import datetime
 import hashlib
@@ -38,10 +37,10 @@ from base64 import b64decode
 import paramiko
 from dogpile.cache import make_region
 from dogpile.cache.api import NO_VALUE
-from sqlalchemy import and_, or_
+from sqlalchemy import and_, or_, delete
 
 from rucio.common.exception import CannotAuthenticate, RucioException
-from rucio.common.utils import generate_uuid, query_bunches
+from rucio.common.utils import generate_uuid
 from rucio.core.account import account_exists
 from rucio.core.oidc import validate_jwt
 from rucio.db.sqla import filter_thread_work
@@ -80,7 +79,7 @@ def get_auth_token_user_pass(account, username, password, appid, ip=None, sessio
     :param ip: IP address of the client a a string.
     :param session: The database session in use.
 
-    :returns: A models.Token object as saved to the database.
+    :returns: A dict with token and expires_at entries.
     """
 
     # Make sure the account exists
@@ -103,18 +102,18 @@ def get_auth_token_user_pass(account, username, password, appid, ip=None, sessio
                                                                         account=account).first()
     db_account = result['account']
 
-    # remove expired tokens
-    session.query(models.Token).filter(models.Token.expired_at < datetime.datetime.utcnow(),
-                                       models.Token.account == account).with_for_update(skip_locked=True).delete()
+    stmt = delete(models.Token).\
+        prefix_with('/*+ INDEX(TOKENS_ACCOUNT_EXPIRED_AT_IDX) */', dialect='oracle').\
+        where(and_(models.Token.expired_at < datetime.datetime.utcnow(), models.Token.account == account))
+    session.execute(stmt)
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
     token = '%(account)s-%(username)s-%(appid)s-%(tuid)s' % locals()
     new_token = models.Token(account=db_account, identity=username, token=token, ip=ip)
     new_token.save(session=session)
-    session.expunge(new_token)
 
-    return new_token
+    return token_dictionary(new_token)
 
 
 @transactional_session
@@ -130,25 +129,25 @@ def get_auth_token_x509(account, dn, appid, ip=None, session=None):
     :param ipaddr: IP address of the client as a string.
     :param session: The database session in use.
 
-    :returns: A models.Token object as saved to the database.
+    :returns: A dict with token and expires_at entries.
     """
 
     # Make sure the account exists
     if not account_exists(account, session=session):
         return None
 
-    # remove expired tokens
-    session.query(models.Token).filter(models.Token.expired_at < datetime.datetime.utcnow(),
-                                       models.Token.account == account).with_for_update(skip_locked=True).delete()
+    stmt = delete(models.Token).\
+        prefix_with('/*+ INDEX(TOKENS_ACCOUNT_EXPIRED_AT_IDX) */', dialect='oracle').\
+        where(and_(models.Token.expired_at < datetime.datetime.utcnow(), models.Token.account == account))
+    session.execute(stmt)
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
     token = '%(account)s-%(dn)s-%(appid)s-%(tuid)s' % locals()
     new_token = models.Token(account=account, identity=dn, token=token, ip=ip)
     new_token.save(session=session)
-    session.expunge(new_token)
 
-    return new_token
+    return token_dictionary(new_token)
 
 
 @transactional_session
@@ -164,25 +163,25 @@ def get_auth_token_gss(account, gsstoken, appid, ip=None, session=None):
     :param ip: IP address of the client as a string.
     :param session: The database session in use.
 
-    :returns: A models.Token object as saved to the database.
+    :returns: A dict with token and expires_at entries.
     """
 
     # Make sure the account exists
     if not account_exists(account, session=session):
         return None
 
-    # remove expired tokens
-    session.query(models.Token).filter(models.Token.expired_at < datetime.datetime.utcnow(),
-                                       models.Token.account == account).with_for_update(skip_locked=True).delete()
+    stmt = delete(models.Token).\
+        prefix_with('/*+ INDEX(TOKENS_ACCOUNT_EXPIRED_AT_IDX) */', dialect='oracle').\
+        where(and_(models.Token.expired_at < datetime.datetime.utcnow(), models.Token.account == account))
+    session.execute(stmt)
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
     token = '%(account)s-%(gsstoken)s-%(appid)s-%(tuid)s' % locals()
     new_token = models.Token(account=account, token=token, ip=ip)
     new_token.save(session=session)
-    session.expunge(new_token)
 
-    return new_token
+    return token_dictionary(new_token)
 
 
 @transactional_session
@@ -198,7 +197,7 @@ def get_auth_token_ssh(account, signature, appid, ip=None, session=None):
     :param ip: IP address of the client as a string.
     :param session: The database session in use.
 
-    :returns: A models.Token object as saved to the database.
+    :returns: A dict with token and expires_at entries.
     """
     if not isinstance(signature, bytes):
         signature = signature.encode()
@@ -237,18 +236,18 @@ def get_auth_token_ssh(account, signature, appid, ip=None, session=None):
     if not match:
         return None
 
-    # remove expired tokens
-    session.query(models.Token).filter(models.Token.expired_at < datetime.datetime.utcnow(),
-                                       models.Token.account == account).with_for_update(skip_locked=True).delete()
+    stmt = delete(models.Token).\
+        prefix_with('/*+ INDEX(TOKENS_ACCOUNT_EXPIRED_AT_IDX) */', dialect='oracle').\
+        where(and_(models.Token.expired_at < datetime.datetime.utcnow(), models.Token.account == account))
+    session.execute(stmt)
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
     token = '%(account)s-ssh:pubkey-%(appid)s-%(tuid)s' % locals()
     new_token = models.Token(account=account, token=token, ip=ip)
     new_token.save(session=session)
-    session.expunge(new_token)
 
-    return new_token
+    return token_dictionary(new_token)
 
 
 @transactional_session
@@ -262,7 +261,7 @@ def get_ssh_challenge_token(account, appid, ip=None, session=None):
     :param appid: The application identifier as a string.
     :param ip: IP address of the client as a string.
 
-    :returns: A models.Token object as saved to the database.
+    :returns: A dict with token and expires_at entries.
     """
 
     # Make sure the account exists
@@ -283,9 +282,8 @@ def get_ssh_challenge_token(account, appid, ip=None, session=None):
     new_challenge_token = models.Token(account=account, token=challenge_token, ip=ip,
                                        expired_at=expiration)
     new_challenge_token.save(session=session)
-    session.expunge(new_challenge_token)
 
-    return new_challenge_token
+    return token_dictionary(new_challenge_token)
 
 
 @transactional_session
@@ -301,24 +299,24 @@ def get_auth_token_saml(account, saml_nameid, appid, ip=None, session=None):
     :param ip: IP address of the client a a string.
     :param session: The database session in use.
 
-    :returns: A models.Token object as saved to the database.
+    :returns: A dict with token and expires_at entries.
     """
 
     # Make sure the account exists
     if not account_exists(account, session=session):
         return None
 
-    # remove expired tokens
-    session.query(models.Token).filter(models.Token.expired_at < datetime.datetime.utcnow(),
-                                       models.Token.account == account).with_for_update(skip_locked=True).delete()
+    stmt = delete(models.Token).\
+        prefix_with('/*+ INDEX(TOKENS_ACCOUNT_EXPIRED_AT_IDX) */', dialect='oracle').\
+        where(and_(models.Token.expired_at < datetime.datetime.utcnow(), models.Token.account == account))
+    session.execute(stmt)
 
     tuid = generate_uuid()  # NOQA
     token = '%(account)s-%(saml_nameid)s-%(appid)s-%(tuid)s' % locals()
     new_token = models.Token(account=account, identity=saml_nameid, token=token, ip=ip)
     new_token.save(session=session)
-    session.expunge(new_token)
 
-    return new_token
+    return token_dictionary(new_token)
 
 
 @transactional_session
@@ -341,18 +339,18 @@ def redirect_auth_oidc(auth_code, fetchtoken=False, session=None):
     try:
         redirect_result = session.query(models.OAuthRequest.redirect_msg).filter_by(access_msg=auth_code).first()
 
-        if isinstance(redirect_result, tuple):
-            if 'http' not in redirect_result[0] and fetchtoken:
-                # in this case the function check if the value is a valid token
-                vdict = validate_auth_token(redirect_result[0], session=session)
-                if vdict:
-                    return redirect_result[0]
-                return None
-            elif 'http' in redirect_result[0] and not fetchtoken:
-                # return redirection URL
+        if not redirect_result:
+            return None
+
+        if 'http' not in redirect_result[0] and fetchtoken:
+            # in this case the function check if the value is a valid token
+            vdict = validate_auth_token(redirect_result[0], session=session)
+            if vdict:
                 return redirect_result[0]
             return None
-        return None
+        elif 'http' in redirect_result[0] and not fetchtoken:
+            # return redirection URL
+            return redirect_result[0]
     except:
         raise CannotAuthenticate(traceback.format_exc())
 
@@ -381,14 +379,14 @@ def delete_expired_tokens(total_workers, worker_number, limit=1000, session=None
         query = filter_thread_work(session=session, query=query, total_threads=total_workers, thread_id=worker_number, hash_variable='token')
 
         # limiting the number of tokens deleted at once
-        filtered_tokens_query = query.limit(limit)
+        query = query.limit(limit)
         # remove expired tokens
         deleted_tokens = 0
-        filtered_bunches = query_bunches(filtered_tokens_query, 10)
-        for items in filtered_bunches:
-            deleted_tokens += session.query(models.Token.token)\
-                                     .filter(models.Token.token.in_(items))\
-                                     .with_for_update(skip_locked=True)\
+        for items in session.execute(query).partitions(10):
+            tokens = tuple(map(lambda row: row.token, items))
+            deleted_tokens += session.query(models.Token) \
+                                     .filter(models.Token.token.in_(tokens)) \
+                                     .with_for_update(skip_locked=True) \
                                      .delete(synchronize_session='fetch')
 
     except Exception as error:
@@ -470,3 +468,7 @@ def validate_auth_token(token, session=None):
         TOKENREGION.delete(token)
         return None
     return value
+
+
+def token_dictionary(token: models.Token):
+    return {'token': token.token, 'expires_at': token.expired_at}
