@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 #
 # Authors:
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2021
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
 # - Yun-Pin Sun <winter0128@gmail.com>, 2013
@@ -32,8 +32,11 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Luc Goossens <luc.goossens@cern.ch>, 2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Vivek Nigam <viveknigam.nigam3@gmail.com>, 2020
+# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 import logging
 import random
@@ -43,7 +46,7 @@ from hashlib import md5
 from re import match
 
 from six import string_types
-from sqlalchemy import and_, or_, exists
+from sqlalchemy import and_, or_, exists, update, delete
 from sqlalchemy.exc import DatabaseError, IntegrityError
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.sql import not_, func
@@ -314,11 +317,13 @@ def __add_files_to_archive(scope, name, files, account, ignore_duplicate=False,
         new_files and session.bulk_insert_mappings(models.DataIdentifier, new_files)
         if existing_files_condition:
             for chunk in chunks(existing_files_condition, 20):
-                session.query(models.DataIdentifier).\
-                    with_hint(models.DataIdentifier, "INDEX(DIDS DIDS_PK)", 'oracle').\
-                    filter(models.DataIdentifier.did_type == DIDType.FILE).\
-                    filter(or_(models.DataIdentifier.constituent.is_(None), models.DataIdentifier.constituent == false())).\
-                    filter(or_(*chunk)).update({'constituent': True})
+                stmt = update(models.DataIdentifier).\
+                    prefix_with("/*+ INDEX(DIDS DIDS_PK) */", dialect='oracle').\
+                    where(models.DataIdentifier.did_type == DIDType.FILE).\
+                    where(or_(models.DataIdentifier.constituent.is_(None), models.DataIdentifier.constituent == false())).\
+                    where(or_(*chunk)).\
+                    values(constituent=True)
+                session.execute(stmt)
         contents and session.bulk_insert_mappings(models.ConstituentAssociation, contents)
         session.flush()
     except IntegrityError as error:
@@ -447,7 +452,7 @@ def __add_collections_to_container(scope, name, collections, account, session):
     :param session: The database session in use.
     """
 
-    condition = or_()
+    condition = []
     for cond in collections:
 
         if (scope == cond['scope']) and (name == cond['name']):
@@ -460,7 +465,7 @@ def __add_collections_to_container(scope, name, collections, account, session):
     child_type = None
     for row in session.query(models.DataIdentifier.scope,
                              models.DataIdentifier.name,
-                             models.DataIdentifier.did_type).with_hint(models.DataIdentifier, "INDEX(DIDS DIDS_PK)", 'oracle').filter(condition):
+                             models.DataIdentifier.did_type).with_hint(models.DataIdentifier, "INDEX(DIDS DIDS_PK)", 'oracle').filter(or_(*condition)):
 
         if row.did_type == DIDType.FILE:
             raise exception.UnsupportedOperation("Adding a file (%s:%s) to a container (%s:%s) is forbidden" % (row.scope, row.name, scope, name))
@@ -605,6 +610,9 @@ def delete_dids(dids, account, expire_rules=False, session=None, logger=logging.
     not_purge_replicas = []
     did_followed_clause = []
     metadata_to_delete = []
+    file_content_clause = []
+
+    archive_dids = config_core.get('undertaker', 'archive_dids', default=False, session=session)
 
     for did in dids:
         logger(logging.INFO, 'Removing did %(scope)s:%(name)s (%(did_type)s)' % did)
@@ -613,6 +621,7 @@ def delete_dids(dids, account, expire_rules=False, session=None, logger=logging.
         else:
             did_clause.append(and_(models.DataIdentifier.scope == did['scope'], models.DataIdentifier.name == did['name']))
             content_clause.append(and_(models.DataIdentifierAssociation.scope == did['scope'], models.DataIdentifierAssociation.name == did['name']))
+            file_content_clause.append(and_(models.DataIdentifierAssociation.scope == did['scope'], models.DataIdentifierAssociation.name == did['name'], models.DataIdentifierAssociation.child_type == DIDType.FILE))
             collection_replica_clause.append(and_(models.CollectionReplica.scope == did['scope'],
                                                   models.CollectionReplica.name == did['name']))
             did_followed_clause.append(and_(models.DidsFollowed.scope == did['scope'], models.DidsFollowed.name == did['name']))
@@ -688,7 +697,19 @@ def delete_dids(dids, account, expire_rules=False, session=None, logger=logging.
         with record_timer_block('undertaker.parent_content'):
             for parent_did in session.query(models.DataIdentifierAssociation).filter(or_(*parent_content_clause)):
                 existing_parent_dids = True
-                detach_dids(scope=parent_did.scope, name=parent_did.name, dids=[{'scope': parent_did.child_scope, 'name': parent_did.child_name}], session=session)
+                detach_dids(scope=parent_did.scope, name=parent_did.name, dids=[{'scope': parent_did.child_scope, 'name': parent_did.child_type}], session=session)
+
+    # Set Epoch tombstone for the files replicas inside the did
+    if config_core.get('undertaker', 'purge_all_replicas', default=False, session=session) and file_content_clause:
+        with record_timer_block('undertaker.file_content'):
+            file_replicas_clause = [and_(models.RSEFileAssociation.scope == cont['child_scope'], models.RSEFileAssociation.name == cont['child_name']) for cont in session.query(models.DataIdentifierAssociation).filter(or_(*file_content_clause))]
+            none_value = None  # Hack to get pep8 happy
+            for chunk in chunks(file_replicas_clause, 100):
+                session.query(models.RSEFileAssociation).filter(or_(*chunk)).\
+                    with_hint(models.RSEFileAssociation, text="INDEX(REPLICAS REPLICAS_PK)", dialect_name='oracle').\
+                    filter(models.RSEFileAssociation.lock_cnt == 0).\
+                    filter(models.RSEFileAssociation.tombstone != none_value).\
+                    update({'tombstone': datetime(1970, 1, 1)}, synchronize_session=False)
 
     # Remove content
     if content_clause:
@@ -727,6 +748,8 @@ def delete_dids(dids, account, expire_rules=False, session=None, logger=logging.
             rowcount = session.query(models.DataIdentifier).filter(or_(*did_clause)).\
                 filter(or_(models.DataIdentifier.did_type == DIDType.CONTAINER, models.DataIdentifier.did_type == DIDType.DATASET)).\
                 delete(synchronize_session=False)
+            if archive_dids:
+                insert_deleted_dids(did_clause, session=session)
 
     if did_followed_clause:
         with record_timer_block('undertaker.dids'):
@@ -847,7 +870,7 @@ def list_new_dids(did_type, thread=None, total_threads=None, chunk_size=1000, se
     """
 
     stmt = select([1]).\
-        prefix_with("/*+ INDEX(RULES ATLAS_RUCIO.RULES_SCOPE_NAME_IDX) */",
+        prefix_with("/*+ INDEX(RULES RULES_SCOPE_NAME_IDX) */",
                     dialect='oracle').\
         where(and_(models.DataIdentifier.scope == models.ReplicationRule.scope,
                    models.DataIdentifier.name == models.ReplicationRule.name,
@@ -1704,11 +1727,10 @@ def resurrect(dids, session=None):
             kargs['expired_at'] = None
         kargs.pop("_sa_instance_state", None)
 
-        session.query(models.DeletedDataIdentifier).\
-            with_hint(models.DeletedDataIdentifier,
-                      "INDEX(DELETED_DIDS DELETED_DIDS_PK)", 'oracle').\
-            filter_by(scope=did['scope'], name=did['name']).\
-            delete()
+        stmt = delete(models.DeletedDataIdentifier).\
+            prefix_with("/*+ INDEX(DELETED_DIDS DELETED_DIDS_PK) */", dialect='oracle').\
+            filter_by(scope=did['scope'], name=did['name'])
+        session.execute(stmt)
 
         models.DataIdentifier(**kargs).\
             save(session=session, flush=False)
