@@ -1,4 +1,5 @@
-# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2018-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,24 +14,24 @@
 # limitations under the License.
 #
 # Authors:
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-#
-# PY3K COMPATIBLE
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2021
 
 from rucio.core import rse as rse_module, distance as distance_module
 from rucio.db.sqla.session import transactional_session
 
 
 @transactional_session
-def export_rses(session=None):
+def export_rses(vo='def', session=None):
     """
     Export RSE data.
 
+    :param vo: The VO to export.
     :param session: database session in use.
     """
     data = {}
-    for rse in rse_module.list_rses(session=session):
+    for rse in rse_module.list_rses(filters={'vo': vo}, session=session):
         rse_id = rse['id']
         data[rse_id] = rse_module.export_rse(rse_id, session=session)
 
@@ -38,14 +39,21 @@ def export_rses(session=None):
 
 
 @transactional_session
-def export_data(session=None):
+def export_data(vo='def', distance=True, session=None):
     """
     Export data.
 
+    :param vo: The VO to export.
+    :param distance: To enable the reporting of distance.
     :param session: database session in use.
     """
-    data = {
-        'rses': export_rses(session=session),
-        'distances': distance_module.export_distances(session=session)
-    }
+    if distance:
+        data = {
+            'rses': export_rses(vo=vo, session=session),
+            'distances': distance_module.export_distances(vo, session=session)
+        }
+    else:
+        data = {
+            'rses': export_rses(vo=vo, session=session)
+        }
     return data
