@@ -17,7 +17,7 @@
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2021
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2017
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2020
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2021
 # - Wen Guan <wen.guan@cern.ch>, 2014-2016
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2016-2021
@@ -28,23 +28,28 @@
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Matt Snyder <msnyder@bnl.gov>, 2021
 # - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Matt Snyder <msnyder@bnl.gov>, 2021
+# - Sahan Dilshan <32576163+sahandilshan@users.noreply.github.com>, 2021
+# - Nick Smith <nick.smith@cern.ch>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 import datetime
 import json
 import logging
 import time
 import traceback
+from collections import namedtuple
 from itertools import filterfalse
-from typing import TYPE_CHECKING, Set
+from typing import TYPE_CHECKING
 
 from six import string_types
 from sqlalchemy import and_, or_, func, update
 from sqlalchemy.exc import IntegrityError
-from sqlalchemy.sql.expression import asc, false, true
+from sqlalchemy.sql.expression import asc, true
 
 from rucio.common.config import config_get_bool
+from rucio.common.constants import FTS_STATE
 from rucio.common.exception import RequestNotFound, RucioException, UnsupportedOperation, ConfigNotFound
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid, chunks, get_parsed_throttler_mode
@@ -53,15 +58,20 @@ from rucio.core.message import add_message
 from rucio.core.monitor import record_counter, record_timer
 from rucio.core.rse import get_rse_name, get_rse_vo, get_rse_transfer_limits, get_rse_attribute
 from rucio.db.sqla import models, filter_thread_work
-from rucio.db.sqla.constants import RequestState, RequestType, FTSState, ReplicaState, LockState, RequestErrMsg
+from rucio.db.sqla.constants import RequestState, RequestType, LockState, RequestErrMsg
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 from rucio.transfertool.fts3 import FTS3Transfertool
 
+RequestAndState = namedtuple('RequestAndState', ['request_id', 'request_state'])
+
 if TYPE_CHECKING:
-    from typing import List, Tuple, Iterable, Iterator, Optional, Callable, Sequence
+    from rucio.core.transfer import RequestWithSources
+    from typing import Any, Dict, Iterable, Iterator, List, Optional, Callable, Set, Union
     from sqlalchemy.orm import Session
 
-    RowIterator = Iterator[Sequence]
+    RequestResult = Dict[str, Any]
+    RequestResultOrState = Union[RequestResult, RequestAndState]
+    RowIterator = Iterator[RequestResult]
     ReduceFunction = Callable[[RowIterator], RowIterator]
 
 """
@@ -400,11 +410,11 @@ def query_request(request_id, transfertool='fts3', session=None, logger=logging.
         else:
             if 'job_state' not in response:
                 req_status['new_state'] = RequestState.LOST
-            elif response['job_state'] in (str(FTSState.FAILED.name),
-                                           str(FTSState.FINISHEDDIRTY.name),
-                                           str(FTSState.CANCELED.name)):
+            elif response['job_state'] in (FTS_STATE.FAILED,
+                                           FTS_STATE.FINISHEDDIRTY,
+                                           FTS_STATE.CANCELED):
                 req_status['new_state'] = RequestState.FAILED
-            elif response['job_state'] == str(FTSState.FINISHED.name):
+            elif response['job_state'] == FTS_STATE.FINISHED:
                 req_status['new_state'] = RequestState.DONE
     else:
         raise NotImplementedError
@@ -477,13 +487,11 @@ def set_request_state(request_id, new_state, transfer_id=None, transferred_at=No
         if err_msg:
             update_items['err_msg'] = err_msg
 
-        if transfer_id:
-            rowcount = session.query(models.Request).filter_by(id=request_id, external_id=transfer_id).update(update_items, synchronize_session=False)
+        request = get_request(request_id, session=session)
+        if new_state in [RequestState.FAILED, RequestState.DONE, RequestState.LOST] and (request["external_id"] != transfer_id):
+            logger(logging.ERROR, "Request %s should not be updated to 'Failed' or 'Done' without external transfer_id" % request_id)
         else:
-            if new_state in [RequestState.FAILED, RequestState.DONE, RequestState.LOST]:
-                logger(logging.ERROR, "Request %s should not be updated to 'Failed' or 'Done' without external transfer_id" % request_id)
-            else:
-                rowcount = session.query(models.Request).filter_by(id=request_id).update(update_items, synchronize_session=False)
+            rowcount = session.query(models.Request).filter_by(id=request_id).update(update_items, synchronize_session=False)
     except IntegrityError as error:
         raise RucioException(error.args)
 
@@ -546,11 +554,13 @@ def touch_requests_by_rule(rule_id, session=None):
     record_counter('core.request.touch_requests_by_rule')
 
     try:
-        session.query(models.Request).with_hint(models.Request, "INDEX(REQUESTS REQUESTS_RULEID_IDX)", 'oracle')\
+        stmt = update(models.Request).prefix_with("/*+ INDEX(REQUESTS REQUESTS_RULEID_IDX) */", dialect='oracle')\
                                      .filter_by(rule_id=rule_id)\
-                                     .filter(models.Request.state.in_([RequestState.FAILED, RequestState.DONE, RequestState.LOST, RequestState.NO_SOURCES, RequestState.ONLY_TAPE_SOURCES]))\
-                                     .filter(models.Request.updated_at < datetime.datetime.utcnow())\
-                                     .update({'updated_at': datetime.datetime.utcnow() + datetime.timedelta(minutes=20)}, synchronize_session=False)
+                                     .where(models.Request.state.in_([RequestState.FAILED, RequestState.DONE, RequestState.LOST, RequestState.NO_SOURCES, RequestState.ONLY_TAPE_SOURCES]))\
+                                     .where(models.Request.updated_at < datetime.datetime.utcnow())\
+                                     .execution_options(synchronize_session=False)\
+                                     .values(updated_at=datetime.datetime.utcnow() + datetime.timedelta(minutes=20))
+        session.execute(stmt)
     except IntegrityError as error:
         raise RucioException(error.args)
 
@@ -657,37 +667,37 @@ def archive_request(request_id, session=None):
     req = get_request(request_id=request_id, session=session)
 
     if req:
-        hist_request = models.Request.__history_mapper__.class_(id=req['id'],
-                                                                created_at=req['created_at'],
-                                                                request_type=req['request_type'],
-                                                                scope=req['scope'],
-                                                                name=req['name'],
-                                                                dest_rse_id=req['dest_rse_id'],
-                                                                source_rse_id=req['source_rse_id'],
-                                                                attributes=req['attributes'],
-                                                                state=req['state'],
-                                                                account=req['account'],
-                                                                external_id=req['external_id'],
-                                                                retry_count=req['retry_count'],
-                                                                err_msg=req['err_msg'],
-                                                                previous_attempt_id=req['previous_attempt_id'],
-                                                                external_host=req['external_host'],
-                                                                rule_id=req['rule_id'],
-                                                                activity=req['activity'],
-                                                                bytes=req['bytes'],
-                                                                md5=req['md5'],
-                                                                adler32=req['adler32'],
-                                                                dest_url=req['dest_url'],
-                                                                requested_at=req['requested_at'],
-                                                                submitted_at=req['submitted_at'],
-                                                                staging_started_at=req['staging_started_at'],
-                                                                staging_finished_at=req['staging_finished_at'],
-                                                                started_at=req['started_at'],
-                                                                estimated_started_at=req['estimated_started_at'],
-                                                                estimated_at=req['estimated_at'],
-                                                                transferred_at=req['transferred_at'],
-                                                                estimated_transferred_at=req['estimated_transferred_at'],
-                                                                transfertool=req['transfertool'])
+        hist_request = models.RequestHistory(id=req['id'],
+                                             created_at=req['created_at'],
+                                             request_type=req['request_type'],
+                                             scope=req['scope'],
+                                             name=req['name'],
+                                             dest_rse_id=req['dest_rse_id'],
+                                             source_rse_id=req['source_rse_id'],
+                                             attributes=req['attributes'],
+                                             state=req['state'],
+                                             account=req['account'],
+                                             external_id=req['external_id'],
+                                             retry_count=req['retry_count'],
+                                             err_msg=req['err_msg'],
+                                             previous_attempt_id=req['previous_attempt_id'],
+                                             external_host=req['external_host'],
+                                             rule_id=req['rule_id'],
+                                             activity=req['activity'],
+                                             bytes=req['bytes'],
+                                             md5=req['md5'],
+                                             adler32=req['adler32'],
+                                             dest_url=req['dest_url'],
+                                             requested_at=req['requested_at'],
+                                             submitted_at=req['submitted_at'],
+                                             staging_started_at=req['staging_started_at'],
+                                             staging_finished_at=req['staging_finished_at'],
+                                             started_at=req['started_at'],
+                                             estimated_started_at=req['estimated_started_at'],
+                                             estimated_at=req['estimated_at'],
+                                             transferred_at=req['transferred_at'],
+                                             estimated_transferred_at=req['estimated_transferred_at'],
+                                             transfertool=req['transfertool'])
         hist_request.save(session=session)
         try:
             time_diff = req['updated_at'] - req['created_at']
@@ -755,94 +765,6 @@ def cancel_request_external_id(transfer_id, transfer_host):
         raise RucioException('Could not cancel FTS3 transfer %s on %s: %s' % (transfer_id, transfer_host, traceback.format_exc()))
 
 
-@read_session
-def list_stagein_requests_and_source_replicas(total_workers=0, worker_number=0, limit=None, activity=None, older_than=None, rses=None, session=None):
-    """
-    List stagein requests with source replicas
-
-    :param total_workers:         Number of total workers.
-    :param worker_number:         Id of the executing worker.
-    :param limit:            Integer of requests to retrieve.
-    :param activity:         Activity to be selected.
-    :param older_than:       Only select requests older than this DateTime.
-    :param rses:             List of rse_id to select requests.
-    :param session:          Database session to use.
-    :returns:                List.
-    """
-    sub_requests = session.query(models.Request.id,
-                                 models.Request.rule_id,
-                                 models.Request.scope,
-                                 models.Request.name,
-                                 models.Request.md5,
-                                 models.Request.adler32,
-                                 models.Request.bytes,
-                                 models.Request.activity,
-                                 models.Request.attributes,
-                                 models.Request.previous_attempt_id,
-                                 models.Request.dest_rse_id,
-                                 models.Request.retry_count)\
-        .with_hint(models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_UPD_IDX)", 'oracle')\
-        .filter(models.Request.state == RequestState.QUEUED)\
-        .filter(models.Request.request_type == RequestType.STAGEIN)
-
-    if isinstance(older_than, datetime.datetime):
-        sub_requests = sub_requests.filter(models.Request.requested_at < older_than)
-
-    if activity:
-        sub_requests = sub_requests.filter(models.Request.activity == activity)
-
-    sub_requests = filter_thread_work(session=session, query=sub_requests, total_threads=total_workers, thread_id=worker_number)
-
-    if limit:
-        sub_requests = sub_requests.limit(limit)
-
-    sub_requests = sub_requests.subquery()
-
-    query = session.query(sub_requests.c.id,
-                          sub_requests.c.rule_id,
-                          sub_requests.c.scope,
-                          sub_requests.c.name,
-                          sub_requests.c.md5,
-                          sub_requests.c.adler32,
-                          sub_requests.c.bytes,
-                          sub_requests.c.activity,
-                          sub_requests.c.attributes,
-                          sub_requests.c.dest_rse_id,
-                          models.RSEFileAssociation.rse_id,
-                          models.RSE.rse,
-                          models.RSE.deterministic,
-                          models.RSE.rse_type,
-                          models.RSEFileAssociation.path,
-                          models.RSEAttrAssociation.value,
-                          sub_requests.c.retry_count,
-                          sub_requests.c.previous_attempt_id,
-                          models.Source.url,
-                          models.Source.ranking)\
-        .outerjoin(models.RSEFileAssociation, and_(sub_requests.c.scope == models.RSEFileAssociation.scope,
-                                                   sub_requests.c.name == models.RSEFileAssociation.name,
-                                                   models.RSEFileAssociation.state == ReplicaState.AVAILABLE,
-                                                   sub_requests.c.dest_rse_id != models.RSEFileAssociation.rse_id))\
-        .with_hint(models.RSEFileAssociation, "+ index(replicas REPLICAS_PK)", 'oracle')\
-        .outerjoin(models.RSE, and_(models.RSE.id == models.RSEFileAssociation.rse_id,
-                                    models.RSE.staging_area == false(),
-                                    models.RSE.deleted == false()))\
-        .outerjoin(models.RSEAttrAssociation, and_(models.RSEAttrAssociation.rse_id == models.RSE.id,
-                                                   models.RSEAttrAssociation.key == 'staging_buffer'))\
-        .outerjoin(models.Source, and_(sub_requests.c.id == models.Source.request_id,
-                                       models.RSE.id == models.Source.rse_id))\
-        .with_hint(models.Source, "+ index(sources SOURCES_PK)", 'oracle')
-
-    if rses:
-        result = []
-        for item in query.all():
-            dest_rse_id = item[9]
-            if dest_rse_id in rses:
-                result.append(item)
-        return result
-    else:
-        return query.all()
-
-
 @read_session
 def get_sources(request_id, rse_id=None, session=None):
     """
@@ -987,8 +909,9 @@ def release_waiting_requests_per_deadline(rse_id=None, deadline=1, session=None)
         old_requests_subquery = session.query(filtered_requests_subquery.c.id)\
                                        .join(old_requests_subquery, and_(filtered_requests_subquery.c.dataset_name == old_requests_subquery.c.name, filtered_requests_subquery.c.dataset_scope == old_requests_subquery.c.scope))
         old_requests_subquery = old_requests_subquery.subquery()
-        statement = update(models.Request).where(models.Request.id.in_(old_requests_subquery)).values(state=RequestState.QUEUED)
-        amount_released_requests = session.execute(statement).rowcount
+        amount_released_requests = session.query(models.Request) \
+            .filter(models.Request.id.in_(old_requests_subquery)) \
+            .update({models.Request.state: RequestState.QUEUED}, synchronize_session=False)
     return amount_released_requests
 
 
@@ -1031,8 +954,9 @@ def release_waiting_requests_per_free_volume(rse_id, volume=None, session=None):
                                        .filter(cumulated_volume_subquery.c.cum_volume <= volume - sum_volume_active_subquery.c.sum_bytes)\
                                        .subquery()
 
-    statement = update(models.Request).where(models.Request.id.in_(cumulated_volume_subquery)).values(state=RequestState.QUEUED)
-    amount_released_requests = session.execute(statement).rowcount
+    amount_released_requests = session.query(models.Request) \
+        .filter(models.Request.id.in_(cumulated_volume_subquery)) \
+        .update({models.Request.state: RequestState.QUEUED}, synchronize_session=False)
     return amount_released_requests
 
 
@@ -1214,9 +1138,9 @@ def release_waiting_requests_grouped_fifo(rse_id, count=None, direction='destina
     # needed for mysql to update and select from the same table
     cumulated_children_subquery = session.query(cumulated_children_subquery.c.id).subquery()
 
-    statement = update(models.Request).where(models.Request.id.in_(cumulated_children_subquery)).values(state=RequestState.QUEUED)
-
-    amount_updated_requests += session.execute(statement).rowcount
+    amount_updated_requests += session.query(models.Request) \
+        .filter(models.Request.id.in_(cumulated_children_subquery)) \
+        .update({models.Request.state: RequestState.QUEUED}, synchronize_session=False)
 
     # release requests where the whole datasets volume fits in the available volume space
     if volume:
@@ -1487,7 +1411,9 @@ def __touch_request(request_id, session=None):
     record_counter('core.request.touch_request')
 
     try:
-        rowcount = session.query(models.Request).filter_by(id=request_id).update({'updated_at': datetime.datetime.utcnow()}, synchronize_session=False)
+        rowcount = session.query(models.Request) \
+            .filter_by(id=request_id) \
+            .update({'updated_at': datetime.datetime.utcnow()}, synchronize_session=False)
     except IntegrityError as error:
         raise RucioException(error.args)
     if not rowcount:
@@ -1543,38 +1469,33 @@ def list_requests(src_rse_ids, dst_rse_ids, states=[RequestState.WAITING], sessi
         yield request
 
 
-# important column indices from __list_transfer_requests_and_source_replicas
-request_id_col = 0
-activity_col = 7
-dest_rse_id_col = 10
-source_rse_id_col = 12
-distance_col = 20
-
-extra_transfertool_col = 21
-
-
 @transactional_session
-def preparer_update_requests(source_iter: "Iterable[Sequence]", session: "Optional[Session]" = None) -> int:
+def preparer_update_requests(source_iter: "Iterable[RequestResultOrState]", session: "Optional[Session]" = None) -> int:
+    """
+    Update transfer requests according to preparer settings.
+    """
     count = 0
-    for req_source in source_iter:
+    for rws in source_iter:
         update_dict = dict()
-        if len(req_source) == 2:
+        if isinstance(rws, RequestAndState):
             # special case where the first entry is the request id and the second is the new state
             # (see handling of RequestState.NO_SOURCES in reduce_requests)
-            update_dict[models.Request.state] = req_source[1]
+            request_id = rws.request_id
+            update_dict[models.Request.state] = rws.request_state
         else:
+            request_id = rws['request_id']
             update_dict[models.Request.state] = __throttler_request_state(
-                activity=req_source[activity_col],
-                source_rse_id=req_source[source_rse_id_col],
-                dest_rse_id=req_source[dest_rse_id_col],
+                activity=rws['activity'],
+                source_rse_id=rws['src_rse_id'],
+                dest_rse_id=rws['dest_rse_id'],
                 session=session,
             )
-            update_dict[models.Request.source_rse_id] = req_source[source_rse_id_col]
+            update_dict[models.Request.source_rse_id] = rws['src_rse_id']
 
-        if len(req_source) > extra_transfertool_col:
-            update_dict[models.Request.transfertool] = req_source[extra_transfertool_col]
+            if 'transfertool' in rws:
+                update_dict[models.Request.transfertool] = rws['transfertool']
 
-        session.query(models.Request).filter_by(id=req_source[request_id_col]).update(update_dict, synchronize_session=False)
+        session.query(models.Request).filter_by(id=request_id).update(update_dict, synchronize_session=False)
         count += 1
     return count
 
@@ -1614,19 +1535,30 @@ def __throttler_request_state(activity, source_rse_id, dest_rse_id, session: "Op
     return RequestState.WAITING if limit_found else RequestState.QUEUED
 
 
-def reduce_requests(req_sources: "List[Tuple]", sort_reduce_funcs: "List[ReduceFunction]", logger: "Callable") -> "RowIterator":
+def reduce_requests(
+    req_sources: "List[RequestWithSources]",
+    sort_reduce_funcs: "List[ReduceFunction]",
+    logger: "Callable",
+) -> "Iterator[RequestResultOrState]":
     """
-    Reduces the passed request sources tuples by using the sort-reduce functions,
-    returning the first of the remaining items or a simple
-    (request_id, RequestState.NO_SOURCES) tuple, if no sources were found.
+    Reduces the passed requests & sources objects by using the sort-reduce
+    functions, yielding the best RequestResult object or a RequestAndState
+    object. If all sources were filtered, a RequestAndState object with
+    RequestState.NO_SOURCES is yielded.
     """
     assert len(req_sources) != 0, 'parameter request sources must be non-empty'
 
-    # sort by Request.id for partitioning later
-    req_sources.sort(key=lambda t: t[request_id_col])
-
-    def pick_result(items: "List[Sequence]") -> "Optional[Sequence]":
-        result = items
+    def pick_result(rws: "RequestWithSources") -> "Optional[RequestResult]":
+        result = [
+            {
+                'request_id': rws.request_id,
+                'dest_rse_id': rws.dest_rse.id,
+                'activity': rws.activity,
+                'src_rse_id': source.rse.id,
+                'distance_ranking': source.distance_ranking
+            }
+            for source in rws.sources
+        ]
         debug_log = []
         for sort_reduce in sort_reduce_funcs:
             newresult = list(sort_reduce(result))
@@ -1634,28 +1566,20 @@ def reduce_requests(req_sources: "List[Tuple]", sort_reduce_funcs: "List[ReduceF
             result = newresult
 
         if len(result) == 0:
-            logger(logging.WARNING, 'all available sources were filtered for requests with id %s', items[0][request_id_col])
+            logger(logging.WARNING, 'all available sources were filtered for requests with id %s', rws.request_id)
             logger(logging.DEBUG, 'the following filters ran:\n' + '\n'.join(debug_log))
         else:
             return result[0]
 
-    def result_or_no_sources(result: "Optional[Sequence]") -> "Sequence":
+    def result_or_no_sources(result: "Optional[RequestResult]") -> "RequestResultOrState":
         if result is None:
-            return (cur_request_id, RequestState.NO_SOURCES)
+            return RequestAndState(request_id=cur_request_id, request_state=RequestState.NO_SOURCES)
         else:
             return result
 
-    cur_request_id = req_sources[0][request_id_col]
-
-    # partition the req_sources by request_id and yield the best result from each group
-    current_items = [req_sources[0]]
-    for idx in range(1, len(req_sources)):
-        if cur_request_id != req_sources[idx][request_id_col]:
-            yield result_or_no_sources(pick_result(current_items))
-            cur_request_id = req_sources[idx][request_id_col]
-            current_items.clear()
-        current_items.append(req_sources[idx])
-    yield result_or_no_sources(pick_result(current_items))
+    for rws in req_sources:
+        cur_request_id = rws.request_id
+        yield result_or_no_sources(pick_result(rws))
 
 
 def get_supported_transfertools(rse_id: str, session=None) -> "Set[str]":
@@ -1674,37 +1598,38 @@ def get_supported_transfertools(rse_id: str, session=None) -> "Set[str]":
 
 
 def get_transfertool_filter(
-        get_transfertools: "Callable[[str], Set[str]]" = get_supported_transfertools
+    get_transfertools: "Callable[[str], Set[str]]" = get_supported_transfertools,
 ) -> "ReduceFunction":
     def filter_requests_for_transfertools(items: "RowIterator") -> "RowIterator":
         first = True
         first_request_id, first_dest_rse_id, dest_rse_transfertools = None, None, None
-        for t in items:
+        for rws_dict in items:
             if first:
                 first = False
-                first_request_id = t[request_id_col]
-                first_dest_rse_id = t[dest_rse_id_col]
+                first_request_id = rws_dict['request_id']
+                first_dest_rse_id = rws_dict['dest_rse_id']
                 dest_rse_transfertools = get_transfertools(first_dest_rse_id)
             else:
                 # same request id, same request destination rse in items per call
-                assert first_request_id == t[request_id_col]
-                assert first_dest_rse_id == t[dest_rse_id_col]
+                assert first_request_id == rws_dict['request_id']
+                assert first_dest_rse_id == rws_dict['dest_rse_id']
 
-            src_rse_transfertools = get_transfertools(t[source_rse_id_col])
+            src_rse_transfertools = get_transfertools(rws_dict['src_rse_id'])
             common_transfertools = dest_rse_transfertools.intersection(src_rse_transfertools)
             if common_transfertools:
                 if 'fts3' in common_transfertools and 'globus' in common_transfertools:
-                    transfertool = 'fts3'
+                    rws_dict['transfertool'] = 'fts3'
                 else:
-                    transfertool = common_transfertools.pop()
-                yield (*t, transfertool)
+                    rws_dict['transfertool'] = common_transfertools.pop()
+                yield rws_dict
 
     return filter_requests_for_transfertools
 
 
 def sort_requests_minimum_distance(items: "RowIterator") -> "RowIterator":
-    yield from sorted(items, key=lambda t: t[distance_col])
+    yield from sorted(items, key=lambda rws_dict: rws_dict['distance_ranking'])
 
 
 def rse_lookup_filter(items: "RowIterator") -> "RowIterator":
-    yield from filter(lambda row: row[source_rse_id_col] is not None and row[dest_rse_id_col] is not None, items)
+    yield from filter(lambda rws_dict: (rws_dict['src_rse_id'] is not None
+                                        and rws_dict['dest_rse_id'] is not None), items)
