@@ -17,8 +17,8 @@
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2015
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2021
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2017
 # - Wen Guan <wen.guan@cern.ch>, 2015-2016
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
@@ -33,41 +33,36 @@
 # - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Tomas Javurek <tomas.javurek@cern.ch>, 2020
 
-from __future__ import division
-
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
-
 import json
+from io import StringIO
+from re import match
+from typing import TYPE_CHECKING
+
 import sqlalchemy
 import sqlalchemy.orm
-
-from re import match
-from six import string_types
-
 from dogpile.cache import make_region
 from dogpile.cache.api import NO_VALUE
-
+from six import string_types
 from sqlalchemy.exc import DatabaseError, IntegrityError, OperationalError
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm.exc import FlushError
 from sqlalchemy.sql.expression import or_, false
 
 import rucio.core.account_counter
-
-from rucio.core.rse_counter import add_counter, get_counter
 from rucio.common import exception, utils
 from rucio.common.config import get_lfn2pfn_algorithm_default, config_get
 from rucio.common.utils import CHECKSUM_KEY, is_checksum_valid, GLOBALLY_SUPPORTED_CHECKSUMS
+from rucio.core.rse_counter import add_counter, get_counter
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import RSEType
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 
+if TYPE_CHECKING:
+    from typing import Dict, Optional
+    from sqlalchemy.orm import Session
 
 REGION = make_region().configure('dogpile.cache.memcached',
                                  expiration_time=3600,
@@ -612,24 +607,35 @@ def get_rse_attribute(key, rse_id=None, value=None, use_cache=True, session=None
 
 
 @read_session
-def get_rse_supported_checksums(rse_id=None, session=None):
+def get_rse_supported_checksums(rse_id, session=None):
     """
-    Retrieve RSE attribute value.
+    Retrieve from the DB and parse the RSE attribute defining the checksum supported by the RSE
+    """
+    return parse_checksum_support_attribute(get_rse_attribute(key=CHECKSUM_KEY, rse_id=rse_id, session=session))
 
-    :param rse_id: The RSE id.
-    :param session: The database session in use.
+
+def get_rse_supported_checksums_from_attributes(rse_attributes):
+    """
+    Parse the RSE attribute defining the checksum supported by the RSE
+    :param rse_attributes: attributes retrieved using list_rse_attributes
+    """
+    return parse_checksum_support_attribute(rse_attributes.get(CHECKSUM_KEY))
+
+
+def parse_checksum_support_attribute(checksum_attribute):
+    """
+    Parse the checksum support RSE attribute.
+    :param checksum_attribute: The value of the RSE attribute storing the checksum value
 
     :returns: The list of checksums supported by the selected RSE.
               If the list is empty (aka attribute is not set) it returns all the default checksums.
               Use 'none' to explicitly tell the RSE does not support any checksum algorithm.
     """
 
-    checksum_support_attribute_list = get_rse_attribute(key=CHECKSUM_KEY, rse_id=rse_id, session=session)
-
-    if not checksum_support_attribute_list:
+    if not checksum_attribute:
         return GLOBALLY_SUPPORTED_CHECKSUMS
     else:
-        supported_checksum_list = checksum_support_attribute_list[0].split(',')
+        supported_checksum_list = checksum_attribute[0].split(',')
         if 'none' in supported_checksum_list:
             return []
         return supported_checksum_list
@@ -719,7 +725,8 @@ def get_rse_usage(rse_id, source=None, session=None, per_account=False):
 
 
 @transactional_session
-def set_rse_limits(rse_id, name, value, session=None):
+def set_rse_limits(rse_id: str, name: str, value: int,
+                   session: 'Session' = None) -> bool:
     """
     Set RSE limits.
 
@@ -737,7 +744,8 @@ def set_rse_limits(rse_id, name, value, session=None):
 
 
 @read_session
-def get_rse_limits(rse_id, name=None, session=None):
+def get_rse_limits(rse_id: str, name: 'Optional[str]' = None,
+                   session: 'Session' = None) -> 'Dict[str, int]':
     """
     Get RSE limits.
 
@@ -750,14 +758,12 @@ def get_rse_limits(rse_id, name=None, session=None):
     query = session.query(models.RSELimit).filter_by(rse_id=rse_id)
     if name:
         query = query.filter_by(name=name)
-    limits = {}
-    for limit in query:
-        limits[limit.name] = limit.value
-    return limits
+    return {limit.name: limit.value for limit in query}
 
 
 @transactional_session
-def delete_rse_limits(rse_id, name=None, session=None):
+def delete_rse_limits(rse_id: str, name: 'Optional[str]' = None,
+                      session: 'Session' = None) -> None:
     """
     Delete RSE limit.
 
@@ -860,7 +866,7 @@ def list_rse_usage_history(rse_id, source=None, session=None):
 
     :returns: A list of historic RSE usage.
     """
-    query = session.query(models.RSEUsage.__history_mapper__.class_).filter_by(rse_id=rse_id).order_by(models.RSEUsage.__history_mapper__.class_.updated_at.desc())
+    query = session.query(models.RSEUsageHistory).filter_by(rse_id=rse_id).order_by(models.RSEUsageHistory.updated_at.desc())  # pylint: disable=no-member
     if source:
         query = query.filter_by(source=source)
 
@@ -944,6 +950,7 @@ def add_protocol(rse_id, parameter, session=None):
                 or match('.*IntegrityError.*ORA-01400: cannot insert NULL into.*RSE_PROTOCOLS.*IMPL.*', error.args[0]) \
                 or match('.*IntegrityError.*Column.*cannot be null.*', error.args[0]) \
                 or match('.*IntegrityError.*null value in column.*violates not-null constraint.*', error.args[0]) \
+                or match('.*IntegrityError.*NOT NULL constraint failed.*', error.args[0]) \
                 or match('.*NotNullViolation.*null value in column.*violates not-null constraint.*', error.args[0]) \
                 or match('.*OperationalError.*cannot be null.*', error.args[0]):
             raise exception.InvalidObject('Missing values!')
@@ -1299,9 +1306,9 @@ def export_rse(rse_id, session=None):
     rse_data.pop('deleted_at')
 
     # get RSE attributes
-    rse_data['attributes'] = list_rse_attributes(rse_id=rse_id)
+    rse_data['attributes'] = list_rse_attributes(rse_id=rse_id, session=session)
 
-    protocols = get_rse_protocols(rse_id=rse_id)
+    protocols = get_rse_protocols(rse_id=rse_id, session=session)
     rse_data['lfn2pfn_algorithm'] = protocols.get('lfn2pfn_algorithm')
     rse_data['verify_checksum'] = protocols.get('verify_checksum')
     rse_data['credentials'] = protocols.get('credentials')
@@ -1311,7 +1318,7 @@ def export_rse(rse_id, session=None):
     rse_data['protocols'] = protocols.get('protocols')
 
     # get RSE limits
-    limits = get_rse_limits(rse_id=rse_id)
+    limits = get_rse_limits(rse_id=rse_id, session=session)
     rse_data['MinFreeSpace'] = limits.get('MinFreeSpace')
     rse_data['MaxBeingDeletedFiles'] = limits.get('MaxBeingDeletedFiles')
 
@@ -1363,7 +1370,7 @@ def delete_qos_policy(rse_id, qos_policy, session=None):
     """
 
     try:
-        session.query(models.RSEQoSAssociation.qos_policy).filter_by(rse_id=rse_id, qos_policy=qos_policy).delete()
+        session.query(models.RSEQoSAssociation).filter_by(rse_id=rse_id, qos_policy=qos_policy).delete()
     except DatabaseError as error:
         raise exception.RucioException(error.args)
 
