@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2020 CERN
+# Copyright 2018-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,9 +15,11 @@
 #
 # Authors:
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 # - Martin Barisits <martin.barisits@cern.ch>, 2020
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2021
 
 """
 Abacus-Collection-Replica is a daemon to update collection replica.
@@ -33,13 +35,14 @@ import traceback
 import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.logging import setup_logging
+from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.replica import get_cleaned_updated_collection_replicas, update_collection_replica
 
 graceful_stop = threading.Event()
 
 
-def collection_replica_update(once=False, limit=1000):
+def collection_replica_update(once=False, limit=1000, sleep_time=10):
     """
     Main loop to check and update the collection replicas.
     """
@@ -70,7 +73,7 @@ def collection_replica_update(once=False, limit=1000):
             # If the list is empty, sent the worker to sleep
             if not replicas and not once:
                 logging.info('collection_replica_update[%s/%s] did not get any work' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1))
-                time.sleep(10)
+                daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=graceful_stop)
             else:
                 for replica in replicas:
                     if graceful_stop.is_set():
@@ -79,7 +82,7 @@ def collection_replica_update(once=False, limit=1000):
                     update_collection_replica(replica)
                     logging.debug('collection_replica_update[%s/%s]: update of collection replica "%s" took %f' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, replica['id'], time.time() - start_time))
                 if limit and len(replicas) < limit and not once:
-                    time.sleep(10)
+                    daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=graceful_stop)
 
         except Exception:
             logging.error(traceback.format_exc())
@@ -99,7 +102,7 @@ def stop(signum=None, frame=None):
     graceful_stop.set()
 
 
-def run(once=False, threads=1):
+def run(once=False, threads=1, sleep_time=10, limit=1000):
     """
     Starts up the Abacus-Collection-Replica threads.
     """
@@ -117,7 +120,8 @@ def run(once=False, threads=1):
         collection_replica_update(once)
     else:
         logging.info('main: starting threads')
-        threads = [threading.Thread(target=collection_replica_update, kwargs={'once': once}) for i in range(0, threads)]
+        threads = [threading.Thread(target=collection_replica_update, kwargs={'once': once, 'sleep_time': sleep_time, 'limit': limit})
+                   for _ in range(0, threads)]
         [t.start() for t in threads]
         logging.info('main: waiting for interrupts')
         # Interruptible joins require a timeout.
