@@ -20,6 +20,7 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 from __future__ import division
 
@@ -35,7 +36,7 @@ import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.exception import DataIdentifierNotFound, ReplicaNotFound
 from rucio.common.logging import setup_logging
-from rucio.common.utils import chunks
+from rucio.common.utils import chunks, daemon_sleep
 from rucio.core import heartbeat
 from rucio.core.did import get_metadata
 from rucio.core.replica import (update_replicas_states, get_replicas_state,
@@ -145,12 +146,9 @@ def minos_tu_expiration(bulk=1000, once=False, sleep_time=60):
         except Exception:
             logging.critical('%s %s', prepend_str, str(traceback.format_exc()))
 
-        tottime = time.time() - start_time
         if once:
             break
-        if tottime < sleep_time:
-            logging.info(prepend_str + 'Will sleep for %s seconds' % (sleep_time - tottime))
-            time.sleep(sleep_time - tottime)
+        daemon_sleep(start_time=start_time, sleep_time=sleep_time, graceful_stop=graceful_stop)
 
     heartbeat.die(executable, hostname, pid, hb_thread)
     logging.info('%s Graceful stop requested', prepend_str)
