@@ -44,6 +44,7 @@ import time
 from collections import defaultdict
 
 from requests.exceptions import RequestException
+from six.moves.configparser import NoOptionError
 from sqlalchemy.exc import DatabaseError
 
 import rucio.db.sqla.util
@@ -55,11 +56,6 @@ from rucio.core import heartbeat, transfer as transfer_core, request as request_
 from rucio.core.monitor import record_timer, record_counter
 from rucio.db.sqla.constants import RequestState, RequestType
 
-try:
-    from ConfigParser import NoOptionError  # py2
-except Exception:
-    from configparser import NoOptionError  # py3
-
 graceful_stop = threading.Event()
 
 datetime.datetime.strptime('', '')
