@@ -23,6 +23,7 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Eric Vaandering <ewv@fnal.gov>, 2019-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 '''
    Hermes is a daemon to deliver messages: to a messagebroker via STOMP, or emails via SMTP.
@@ -48,6 +49,7 @@ import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.config import config_get, config_get_int, config_get_bool
 from rucio.common.logging import setup_logging
+from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.message import retrieve_messages, delete_messages
 from rucio.core.monitor import record_counter
@@ -60,12 +62,15 @@ GRACEFUL_STOP = threading.Event()
 RECONNECT_COUNTER = Counter('rucio_daemons_hermes_reconnect', 'Counts Hermes reconnects to different ActiveMQ brokers', labelnames=('host',))
 
 
-def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=10):
+def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=60, sleep_time=60):
     '''
     Main loop to deliver emails via SMTP.
     '''
     logging.info('[email] starting - threads (%i) bulk (%i)', thread, bulk)
 
+    if sleep_time == deliver_emails.__defaults__[5] and delay != deliver_emails.__defaults__[4]:
+        sleep_time = delay
+
     executable = 'hermes [email]'
     hostname = socket.getfqdn()
     pid = os.getpid()
@@ -134,12 +139,10 @@ def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=10):
         if once:
             break
 
-        t_delay = delay - (time.time() - t_start)
-        t_delay = t_delay if t_delay > 0 else 0
-        if t_delay:
-            logging.debug('[email] %i:%i - sleeping %s seconds',
-                          heartbeat['assign_thread'], heartbeat['nr_threads'], t_delay)
-        time.sleep(t_delay)
+        if len(messages) < bulk:
+            logging.info("Only %d messages, which is less than the bulk %d, will sleep"
+                         % (len(messages), bulk))
+            daemon_sleep(start_time=t_start, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
 
     logging.debug('[email] %i:%i - graceful stop requested', heartbeat['assign_thread'],
                   heartbeat['nr_threads'])
@@ -167,13 +170,16 @@ class HermesListener(stomp.ConnectionListener):
         logging.error('[broker] [%s]: %s', self.__broker, frame.body)
 
 
-def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, delay=10,
-                     broker_timeout=3, broker_retry=3):
+def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, delay=60,
+                     broker_timeout=3, broker_retry=3, sleep_time=60):
     '''
     Main loop to deliver messages to a broker.
     '''
     logging.info('[broker] starting - threads (%i) bulk (%i)', thread, bulk)
 
+    if sleep_time == deliver_messages.__defaults__[7] and delay != deliver_messages.__defaults__[4]:
+        sleep_time = delay
+
     if not brokers_resolved:
         logging.fatal('No brokers resolved.')
         return
@@ -352,19 +358,17 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
                 if once:
                     break
 
+            if len(messages) < bulk:
+                logging.info("Only %d messages, which is less than the bulk %d, will sleep"
+                             % (len(messages), bulk))
+                daemon_sleep(start_time=t_start, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
+
         except NoResultFound:
             # silence this error: https://its.cern.ch/jira/browse/RUCIO-1699
             pass
         except:
             logging.critical(traceback.format_exc())
 
-        t_delay = delay - (time.time() - t_start)
-        t_delay = t_delay if t_delay > 0 else 0
-        if t_delay:
-            logging.debug('[broker] %i:%i - sleeping %s seconds',
-                          heartbeat['assign_thread'], heartbeat['nr_threads'], t_delay)
-        time.sleep(t_delay)
-
     for conn in conns:
         try:
             conn.disconnect()
@@ -388,8 +392,8 @@ def stop(signum=None, frame=None):
     GRACEFUL_STOP.set()
 
 
-def run(once=False, send_email=True, threads=1, bulk=1000, delay=10, broker_timeout=3,
-        broker_retry=3):
+def run(once=False, send_email=True, threads=1, bulk=1000, delay=60, broker_timeout=3,
+        broker_retry=3, sleep_time=60):
     '''
     Starts up the hermes threads.
     '''
@@ -424,9 +428,9 @@ def run(once=False, send_email=True, threads=1, bulk=1000, delay=10, broker_time
         deliver_messages(once=once,
                          brokers_resolved=brokers_resolved,
                          bulk=bulk, delay=delay,
-                         broker_timeout=broker_timeout, broker_retry=broker_retry)
+                         broker_timeout=broker_timeout, broker_retry=broker_retry, sleep_time=sleep_time)
         deliver_emails(once=once,
-                       send_email=send_email, bulk=bulk, delay=delay)
+                       send_email=send_email, bulk=bulk, delay=delay, sleep_time=sleep_time)
 
     else:
         logging.info('starting hermes threads')
@@ -435,12 +439,14 @@ def run(once=False, send_email=True, threads=1, bulk=1000, delay=10, broker_time
                                                                          'bulk': bulk,
                                                                          'delay': delay,
                                                                          'broker_timeout': broker_timeout,
-                                                                         'broker_retry': broker_retry}) for i in range(0, threads)]
+                                                                         'broker_retry': broker_retry,
+                                                                         'sleep_time': sleep_time}) for i in range(0, threads)]
 
         for thrd in range(0, 1):
             thread_list.append(threading.Thread(target=deliver_emails, kwargs={'thread': thrd,
                                                                                'bulk': bulk,
-                                                                               'delay': delay}))
+                                                                               'delay': delay,
+                                                                               'sleep_time': sleep_time}))
 
         for thrd in thread_list:
             thrd.start()
