@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2015
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2018
@@ -22,6 +22,7 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 """
 Judge-Cleaner is a daemon to clean expired replication rules.
@@ -43,6 +44,7 @@ import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.logging import formatted_logger, setup_logging
 from rucio.common.exception import DatabaseException, UnsupportedOperation, RuleNotFound
+from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.monitor import record_counter
 from rucio.core.rule import delete_rule, get_expired_rules
@@ -51,7 +53,7 @@ from rucio.db.sqla.util import get_db_time
 graceful_stop = threading.Event()
 
 
-def rule_cleaner(once=False):
+def rule_cleaner(once=False, sleep_time=60):
     """
     Main loop to check for expired replication rules
     """
@@ -87,12 +89,12 @@ def rule_cleaner(once=False):
             rules = get_expired_rules(total_workers=heartbeat['nr_threads'],
                                       worker_number=heartbeat['assign_thread'],
                                       limit=200,
-                                      blacklisted_rules=[key for key in paused_rules])
+                                      blocked_rules=[key for key in paused_rules])
             logger(logging.DEBUG, 'index query time %f fetch size is %d' % (time.time() - start, len(rules)))
 
             if not rules and not once:
                 logger(logging.DEBUG, 'did not get any work (paused_rules=%s)' % str(len(paused_rules)))
-                graceful_stop.wait(60)
+                daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=graceful_stop, logger=logger)
             else:
                 for rule in rules:
                     rule_id = rule[0]
@@ -146,7 +148,7 @@ def stop(signum=None, frame=None):
     graceful_stop.set()
 
 
-def run(once=False, threads=1):
+def run(once=False, threads=1, sleep_time=60):
     """
     Starts up the Judge-Clean threads.
     """
@@ -170,7 +172,8 @@ def run(once=False, threads=1):
         rule_cleaner(once)
     else:
         logging.info('Cleaner starting %s threads' % str(threads))
-        threads = [threading.Thread(target=rule_cleaner, kwargs={'once': once}) for i in range(0, threads)]
+        threads = [threading.Thread(target=rule_cleaner, kwargs={'once': once,
+                                                                 'sleep_time': sleep_time}) for i in range(0, threads)]
         [t.start() for t in threads]
         # Interruptible joins require a timeout.
         while threads[0].is_alive():
