@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2016-2018
@@ -23,6 +23,7 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 """
 Judge-Evaluator is a daemon to re-evaluate and execute replication rules.
@@ -46,6 +47,7 @@ import rucio.db.sqla.util
 from rucio.common.exception import DatabaseException, DataIdentifierNotFound, ReplicationRuleCreationTemporaryFailed
 from rucio.common.logging import setup_logging
 from rucio.common.types import InternalScope
+from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.monitor import record_counter
 from rucio.core.rule import re_evaluate_did, get_updated_dids, delete_updated_did
@@ -53,7 +55,7 @@ from rucio.core.rule import re_evaluate_did, get_updated_dids, delete_updated_di
 graceful_stop = threading.Event()
 
 
-def re_evaluator(once=False):
+def re_evaluator(once=False, sleep_time=30, did_limit=100):
     """
     Main loop to check the re-evaluation of dids.
     """
@@ -82,18 +84,18 @@ def re_evaluator(once=False):
             # Select a bunch of dids for re evaluation for this worker
             dids = get_updated_dids(total_workers=heartbeat['nr_threads'],
                                     worker_number=heartbeat['assign_thread'],
-                                    limit=100,
-                                    blacklisted_dids=[(InternalScope(key[0], fromExternal=False), key[1]) for key in paused_dids])
-            logging.debug('re_evaluator[%s/%s] index query time %f fetch size is %d (%d blacklisted)' % (heartbeat['assign_thread'],
-                                                                                                         heartbeat['nr_threads'],
-                                                                                                         time.time() - start,
-                                                                                                         len(dids),
-                                                                                                         len([(InternalScope(key[0], fromExternal=False), key[1]) for key in paused_dids])))
+                                    limit=did_limit,
+                                    blocked_dids=[(InternalScope(key[0], fromExternal=False), key[1]) for key in paused_dids])
+            logging.debug('re_evaluator[%s/%s] index query time %f fetch size is %d (%d blocked)' % (heartbeat['assign_thread'],
+                                                                                                     heartbeat['nr_threads'],
+                                                                                                     time.time() - start,
+                                                                                                     len(dids),
+                                                                                                     len([(InternalScope(key[0], fromExternal=False), key[1]) for key in paused_dids])))
 
             # If the list is empty, sent the worker to sleep
             if not dids and not once:
                 logging.debug('re_evaluator[%s/%s] did not get any work (paused_dids=%s)' % (heartbeat['assign_thread'], heartbeat['nr_threads'], str(len(paused_dids))))
-                graceful_stop.wait(30)
+                daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=graceful_stop)
             else:
                 done_dids = {}
                 for did in dids:
@@ -170,7 +172,7 @@ def stop(signum=None, frame=None):
     graceful_stop.set()
 
 
-def run(once=False, threads=1):
+def run(once=False, threads=1, sleep_time=30, did_limit=100):
     """
     Starts up the Judge-Eval threads.
     """
@@ -184,10 +186,12 @@ def run(once=False, threads=1):
     sanity_check(executable=executable, hostname=hostname)
 
     if once:
-        re_evaluator(once)
+        re_evaluator(once=once, did_limit=did_limit)
     else:
         logging.info('Evaluator starting %s threads' % str(threads))
-        threads = [threading.Thread(target=re_evaluator, kwargs={'once': once}) for i in range(0, threads)]
+        threads = [threading.Thread(target=re_evaluator, kwargs={'once': once,
+                                                                 'sleep_time': sleep_time,
+                                                                 'did_limit': did_limit}) for i in range(0, threads)]
         [t.start() for t in threads]
         # Interruptible joins require a timeout.
         while threads[0].is_alive():
