@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2020 CERN
+# Copyright 2015-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,13 +14,14 @@
 # limitations under the License.
 #
 # Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2017
+# - Martin Barisits <martin.barisits@cern.ch>, 2015-2021
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 # - Eric Vaandering <ewv@fnal.gov>, 2020
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 """
 Judge-Injector is a daemon to asynchronously create replication rules
@@ -40,8 +41,9 @@ from sqlalchemy.exc import DatabaseError
 
 import rucio.db.sqla.util
 from rucio.common.logging import formatted_logger, setup_logging
-from rucio.common.exception import (DatabaseException, RuleNotFound, RSEBlacklisted, RSEWriteBlocked,
+from rucio.common.exception import (DatabaseException, RuleNotFound, RSEWriteBlocked,
                                     ReplicationRuleCreationTemporaryFailed, InsufficientAccountLimit)
+from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.monitor import record_counter
 from rucio.core.rule import inject_rule, get_injected_rules, update_rule
@@ -49,7 +51,7 @@ from rucio.core.rule import inject_rule, get_injected_rules, update_rule
 graceful_stop = threading.Event()
 
 
-def rule_injector(once=False):
+def rule_injector(once=False, sleep_time=60):
     """
     Main loop to check for asynchronous creation of replication rules
     """
@@ -85,12 +87,12 @@ def rule_injector(once=False):
             rules = get_injected_rules(total_workers=heartbeat['nr_threads'],
                                        worker_number=heartbeat['assign_thread'],
                                        limit=100,
-                                       blacklisted_rules=[key for key in paused_rules])
+                                       blocked_rules=[key for key in paused_rules])
             logger(logging.DEBUG, 'index query time %f fetch size is %d' % (time.time() - start, len(rules)))
 
             if not rules and not once:
                 logger(logging.DEBUG, 'did not get any work (paused_rules=%s)' % str(len(paused_rules)))
-                graceful_stop.wait(60)
+                daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=graceful_stop, logger=logger)
             else:
                 for rule in rules:
                     rule_id = rule[0]
@@ -115,9 +117,9 @@ def rule_injector(once=False):
                         else:
                             logger(logging.ERROR, 'DatabaseException', exc_info=True)
                             record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
-                    except (RSEBlacklisted, RSEWriteBlocked) as e:
+                    except (RSEWriteBlocked) as e:
                         paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
-                        logger(logging.WARNING, 'RSEBlacklisted for rule %s' % rule_id)
+                        logger(logging.WARNING, 'RSEWriteBlocked for rule %s' % rule_id)
                         record_counter('rule.judge.exceptions.%s' % e.__class__.__name__)
                     except ReplicationRuleCreationTemporaryFailed as e:
                         paused_rules[rule_id] = datetime.utcnow() + timedelta(seconds=randint(60, 600))
@@ -158,7 +160,7 @@ def stop(signum=None, frame=None):
     graceful_stop.set()
 
 
-def run(once=False, threads=1):
+def run(once=False, threads=1, sleep_time=60):
     """
     Starts up the Judge-Injector threads.
     """
@@ -175,7 +177,8 @@ def run(once=False, threads=1):
         rule_injector(once)
     else:
         logging.info('Injector starting %s threads' % str(threads))
-        threads = [threading.Thread(target=rule_injector, kwargs={'once': once}) for i in range(0, threads)]
+        threads = [threading.Thread(target=rule_injector, kwargs={'once': once,
+                                                                  'sleep_time': sleep_time}) for i in range(0, threads)]
         [t.start() for t in threads]
         # Interruptible joins require a timeout.
         while threads[0].is_alive():
