@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2020 CERN
+# Copyright 2013-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,13 +14,14 @@
 # limitations under the License.
 #
 # Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2016
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2018
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2015
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2020
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 """
 Judge-Repairer is a daemon to repair stuck replication rules.
@@ -43,6 +44,7 @@ import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.exception import DatabaseException
 from rucio.common.logging import setup_logging
+from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.monitor import record_counter
 from rucio.core.rule import repair_rule, get_stuck_rules
@@ -50,7 +52,7 @@ from rucio.core.rule import repair_rule, get_stuck_rules
 graceful_stop = threading.Event()
 
 
-def rule_repairer(once=False):
+def rule_repairer(once=False, sleep_time=60):
     """
     Main loop to check for STUCK replication rules
     """
@@ -84,13 +86,13 @@ def rule_repairer(once=False):
                                     worker_number=heartbeat['assign_thread'],
                                     delta=-1 if once else 1800,
                                     limit=100,
-                                    blacklisted_rules=[key for key in paused_rules])
+                                    blocked_rules=[key for key in paused_rules])
 
             logging.debug('rule_repairer[%s/%s] index query time %f fetch size is %d' % (heartbeat['assign_thread'], heartbeat['nr_threads'], time.time() - start, len(rules)))
 
             if not rules and not once:
                 logging.debug('rule_repairer[%s/%s] did not get any work (paused_rules=%s)' % (heartbeat['assign_thread'], heartbeat['nr_threads'], str(len(paused_rules))))
-                graceful_stop.wait(60)
+                daemon_sleep(start, sleep_time, graceful_stop)
             else:
                 for rule_id in rules:
                     rule_id = rule_id[0]
@@ -143,7 +145,7 @@ def stop(signum=None, frame=None):
     graceful_stop.set()
 
 
-def run(once=False, threads=1):
+def run(once=False, threads=1, sleep_time=60):
     """
     Starts up the Judge-Repairer threads.
     """
@@ -160,7 +162,8 @@ def run(once=False, threads=1):
         rule_repairer(once)
     else:
         logging.info('Repairer starting %s threads' % str(threads))
-        threads = [threading.Thread(target=rule_repairer, kwargs={'once': once}) for i in range(0, threads)]
+        threads = [threading.Thread(target=rule_repairer, kwargs={'once': once,
+                                                                  'sleep_time': sleep_time}) for i in range(0, threads)]
         [t.start() for t in threads]
         # Interruptible joins require a timeout.
         while threads[0].is_alive():
