@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2019-2020 CERN
+# Copyright 2019-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,8 @@
 # Authors:
 # - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 """
 OAuth Manager is a daemon which is reponsible for:
@@ -46,6 +47,7 @@ from sqlalchemy.exc import DatabaseError
 import rucio.db.sqla.util
 from rucio.common.exception import DatabaseException
 from rucio.common.logging import setup_logging
+from rucio.common.utils import daemon_sleep
 from rucio.core.authentication import delete_expired_tokens
 from rucio.core.heartbeat import die, live, sanity_check
 from rucio.core.monitor import record_counter, record_timer
@@ -54,20 +56,25 @@ from rucio.core.oidc import delete_expired_oauthrequests, refresh_jwt_tokens
 GRACEFUL_STOP = threading.Event()
 
 
-def OAuthManager(once=False, loop_rate=300, max_rows=100):
+def OAuthManager(once=False, loop_rate=300, max_rows=100, sleep_time=300):
     """
     Main loop to delete all expired tokens, refresh tokens eligible
     for refresh and delete all expired OAuth session parameters.
     It was decided to have only 1 daemon for all 3 of these cleanup activities.
 
     :param once: If True, the loop is run just once, otherwise the daemon continues looping until stopped.
-    :param loop_rate: The number of seconds the daemon will wait before running next loop of operations.
+    :param loop_rate: obsolete, please use sleep_time instead. The number of seconds the daemon will wait before running next loop of operations.
     :param max_rows: Max number of DB rows to deal with per operation.
+    :param sleep_time: The number of seconds the daemon will wait before running next loop of operations.
 
     :returns: None
     """
+    if sleep_time == OAuthManager.__defaults__[3] and loop_rate != OAuthManager.__defaults__[1]:
+        sleep_time = loop_rate
 
     executable = 'oauth-manager'
+    total_workers = 0
+    worker_number = 0
 
     sanity_check(executable=executable, hostname=socket.gethostname())
 
@@ -91,7 +98,7 @@ def OAuthManager(once=False, loop_rate=300, max_rows=100):
             # ACCESS TOKEN REFRESH - better to run first (in case some of the refreshed tokens needed deletion after this step)
             logging.info('oauth_manager[%i/%i]: ----- START ----- ACCESS TOKEN REFRESH ----- ', worker_number, total_workers)
             logging.info('oauth_manager[%i/%i]: starting to query tokens for automatic refresh', worker_number, total_workers)
-            nrefreshed = refresh_jwt_tokens(total_workers, worker_number, refreshrate=int(loop_rate), limit=max_rows)
+            nrefreshed = refresh_jwt_tokens(total_workers, worker_number, refreshrate=int(sleep_time), limit=max_rows)
             logging.info('oauth_manager[%i/%i]: successfully refreshed %i tokens', worker_number, total_workers, nrefreshed)
             logging.info('oauth_manager[%i/%i]: ----- END ----- ACCESS TOKEN REFRESH ----- ', worker_number, total_workers)
             record_counter(counters='oauth_manager.tokens.refreshed', delta=nrefreshed)
@@ -99,16 +106,16 @@ def OAuthManager(once=False, loop_rate=300, max_rows=100):
         except (DatabaseException, DatabaseError) as err:
             if match('.*QueuePool.*', str(err.args[0])):
                 logging.warning(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
             elif match('.*ORA-03135.*', str(err.args[0])):
                 logging.warning(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
             else:
                 logging.critical(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
         except Exception as err:
             logging.critical(traceback.format_exc())
-            record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+            record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
 
         try:
             # waiting 1 sec as DBs does not store milisecond and tokens
@@ -125,16 +132,16 @@ def OAuthManager(once=False, loop_rate=300, max_rows=100):
         except (DatabaseException, DatabaseError) as err:
             if match('.*QueuePool.*', str(err.args[0])):
                 logging.warning(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
             elif match('.*ORA-03135.*', str(err.args[0])):
                 logging.warning(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
             else:
                 logging.critical(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
         except Exception as err:
             logging.critical(traceback.format_exc())
-            record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+            record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
 
         try:
             # DELETING EXPIRED OAUTH SESSION PARAMETERS
@@ -148,16 +155,16 @@ def OAuthManager(once=False, loop_rate=300, max_rows=100):
         except (DatabaseException, DatabaseError) as err:
             if match('.*QueuePool.*', str(err.args[0])):
                 logging.warning(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
             elif match('.*ORA-03135.*', str(err.args[0])):
                 logging.warning(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
             else:
                 logging.critical(traceback.format_exc())
-                record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+                record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
         except Exception as err:
             logging.critical(traceback.format_exc())
-            record_counter('oauth_manager.exceptions.%s', err.__class__.__name__)
+            record_counter('oauth_manager.exceptions.' + err.__class__.__name__)
 
         tottime = time.time() - start
         logging.info('oauth_manager[%i/%i]: took %f seconds to delete %i tokens, %i session parameters and refreshed %i tokens', worker_number, total_workers, tottime, ndeleted, ndeletedreq, nrefreshed)
@@ -166,14 +173,13 @@ def OAuthManager(once=False, loop_rate=300, max_rows=100):
         if once:
             break
         else:
-            logging.info('oauth_manager[%i/%i]: Sleeping for %i seconds.', worker_number, total_workers, loop_rate)
-            GRACEFUL_STOP.wait(loop_rate)
+            daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
 
     die(executable=executable, hostname=socket.gethostname(), pid=os.getpid(), thread=threading.current_thread())
     logging.info('oauth_manager[%i/%i]: graceful stop done', worker_number, total_workers)
 
 
-def run(once=False, threads=1, loop_rate=300, max_rows=100):
+def run(once=False, threads=1, loop_rate=300, max_rows=100, sleep_time=300):
     """
     Starts up the OAuth Manager threads.
     """
@@ -185,13 +191,14 @@ def run(once=False, threads=1, loop_rate=300, max_rows=100):
     sanity_check(executable='OAuthManager', hostname=socket.gethostname())
 
     if once:
-        OAuthManager(once, loop_rate, max_rows)
+        OAuthManager(once, loop_rate, max_rows, sleep_time)
     else:
         logging.info('OAuth Manager starting %s threads', str(threads))
         threads = [threading.Thread(target=OAuthManager,
                                     kwargs={'once': once,
                                             'loop_rate': int(loop_rate),
-                                            'max_rows': max_rows}) for i in range(0, threads)]
+                                            'max_rows': max_rows,
+                                            'sleep_time': sleep_time}) for i in range(0, threads)]
         [t.start() for t in threads]
         # Interruptible joins require a timeout.
         while threads[0].is_alive():
