@@ -16,11 +16,11 @@
 # Authors:
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 # - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Mayank Sharma <mayank.sharma@cern.ch>, 2021
+# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 
 from __future__ import print_function
 
-import importlib
-import os
 import traceback
 
 import pytest
@@ -31,12 +31,14 @@ import pytest
 
 @pytest.fixture(scope='session')
 def vo():
-    from rucio.common.config import config_get_bool, config_get
+    from rucio.tests.common_server import get_vo
+    return get_vo()
 
-    if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
-        return config_get('client', 'vo', raise_exception=False, default='tst')
-    else:
-        return 'def'
+
+@pytest.fixture(scope='session')
+def long_vo():
+    from rucio.tests.common import get_long_vo
+    return get_long_vo()
 
 
 @pytest.fixture(scope='module')
@@ -46,6 +48,12 @@ def replica_client():
     return ReplicaClient()
 
 
+@pytest.fixture(scope='module')
+def rucio_client():
+    from rucio.client import Client
+    return Client()
+
+
 @pytest.fixture(scope='module')
 def did_client():
     from rucio.client.didclient import DIDClient
@@ -57,88 +65,36 @@ def did_client():
 def rest_client():
     from rucio.tests.common import print_response
 
-    backend = os.environ.get('REST_BACKEND', 'webpy')
-    if backend == 'flask':
-        from flask.testing import FlaskClient
-        from rucio.web.rest.flaskapi.v1.main import application
+    from flask.testing import FlaskClient
+    from rucio.web.rest.flaskapi.v1.main import application
 
-        class WrappedFlaskClient(FlaskClient):
-            def __init__(self, *args, **kwargs):
-                super(WrappedFlaskClient, self).__init__(*args, **kwargs)
+    class WrappedFlaskClient(FlaskClient):
+        def __init__(self, *args, **kwargs):
+            super(WrappedFlaskClient, self).__init__(*args, **kwargs)
 
-            def open(self, path='/', *args, **kwargs):
-                print(kwargs.get('method', 'GET'), path)
-                response = super(WrappedFlaskClient, self).open(path, *args, **kwargs)
-                try:
-                    print_response(response)
-                except Exception:
-                    traceback.print_exc()
-                return response
+        def open(self, path='/', *args, **kwargs):
+            print(kwargs.get('method', 'GET'), path)
+            response = super(WrappedFlaskClient, self).open(path, *args, **kwargs)
+            try:
+                print_response(response)
+            except Exception:
+                traceback.print_exc()
+            return response
 
-        _testing = application.testing
-        application.testing = True
-        application.test_client_class = WrappedFlaskClient
-        with application.test_client() as client:
-            yield client
-        application.test_client_class = None
-        application.testing = _testing
-    elif backend == 'webpy':
-        from werkzeug.test import Client as TestClient
-        from werkzeug.wrappers import BaseResponse
-        from rucio.web.rest.main import application as main_application
-
-        def _path_matches_endpoint(path, endpoint):
-            return path.startswith(endpoint + '/') or path.startswith(endpoint + '?') or path == endpoint
-
-        class WrappedTestClient(TestClient):
-            special_endpoints = {
-                '/auth': ('rucio.web.rest.authentication', None),
-                '/credentials': ('rucio.web.rest.credential', None),
-                '/nongrid_traces': ('rucio.web.rest.nongrid_trace', None),
-                '/ping': ('rucio.web.rest.ping', None),
-                '/redirect': ('rucio.web.rest.redirect', None),
-                '/traces': ('rucio.web.rest.trace', None),
-            }
-
-            def __init__(self, *args, **kwargs):
-                super(WrappedTestClient, self).__init__(*args, **kwargs)
-
-            def _endpoint_specials(self, path):
-                for endpoint_path in self.special_endpoints:
-                    if _path_matches_endpoint(path, endpoint_path):
-                        endpoint_module, endpoint_client = self.special_endpoints[endpoint_path]
-                        if endpoint_client is None:
-                            module = importlib.import_module(endpoint_module)
-                            endpoint_client = TestClient(getattr(module, 'application'), BaseResponse)
-                            self.special_endpoints[endpoint_path] = (module, endpoint_client)
-
-                        path = path[len(endpoint_path):]
-                        if not path.startswith('/'):
-                            path = '/' + path
-
-                        return path, endpoint_client.open
-                return path, super(WrappedTestClient, self).open
-
-            def open(self, path='/', *args, **kwargs):
-                newpath, open_method = self._endpoint_specials(path)
-                print(kwargs.get('method', 'GET'), path)
-                response = open_method(newpath, *args, **kwargs)
-                try:
-                    print_response(response)
-                except Exception:
-                    traceback.print_exc()
-                return response
-
-        yield WrappedTestClient(main_application, BaseResponse)
-    else:
-        raise RuntimeError('Unknown rest backend ' + backend)
+    _testing = application.testing
+    application.testing = True
+    application.test_client_class = WrappedFlaskClient
+    with application.test_client() as client:
+        yield client
+    application.test_client_class = None
+    application.testing = _testing
 
 
 @pytest.fixture
-def auth_token(rest_client, vo):
+def auth_token(rest_client, long_vo):
     from rucio.tests.common import vohdr, headers, loginhdr
 
-    auth_response = rest_client.get('/auth/userpass', headers=headers(loginhdr('root', 'ddmlab', 'secret'), vohdr(vo)))
+    auth_response = rest_client.get('/auth/userpass', headers=headers(loginhdr('root', 'ddmlab', 'secret'), vohdr(long_vo)))
     assert auth_response.status_code == 200
     token = auth_response.headers.get('X-Rucio-Auth-Token')
     assert token
@@ -152,6 +108,13 @@ def mock_scope(vo):
     return InternalScope('mock', vo=vo)
 
 
+@pytest.fixture(scope='module')
+def test_scope(vo):
+    from rucio.common.types import InternalScope
+
+    return InternalScope('test', vo=vo)
+
+
 @pytest.fixture(scope='module')
 def root_account(vo):
     from rucio.common.types import InternalAccount
@@ -159,6 +122,28 @@ def root_account(vo):
     return InternalAccount('root', vo=vo)
 
 
+@pytest.fixture(scope="module")
+def containerized_rses(rucio_client):
+    """
+    Detects if containerized rses for xrootd are available in the testing environment.
+    :return: A list of (rse_name, rse_id) tuples.
+    """
+    from rucio.common.exception import InvalidRSEExpression
+
+    rses = []
+    try:
+        xrd_rses = [x['rse'] for x in rucio_client.list_rses(rse_expression='test_container_xrd=True')]
+        xrd_rses = [rucio_client.get_rse(rse) for rse in xrd_rses]
+        xrd_containerized_rses = [(rse_obj['rse'], rse_obj['id']) for rse_obj in xrd_rses if "xrd" in rse_obj['rse'].lower()]
+        xrd_containerized_rses.sort()
+        rses.extend(xrd_containerized_rses)
+    except InvalidRSEExpression as invalid_rse_expression:
+        print("{ex}. Note that containerized RSEs will not be available in non-containerized test environments"
+              .format(ex=invalid_rse_expression))
+        traceback.print_exc()
+    return rses
+
+
 @pytest.fixture
 def rse_factory(vo):
     from rucio.tests.temp_factories import TemporaryRSEFactory
@@ -175,6 +160,14 @@ def did_factory(vo, mock_scope):
         yield factory
 
 
+@pytest.fixture
+def file_factory(tmp_path_factory):
+    from rucio.tests.temp_factories import TemporaryFileFactory
+
+    with TemporaryFileFactory(pytest_path_factory=tmp_path_factory) as factory:
+        yield factory
+
+
 def __get_fixture_param(request):
     fixture_param = getattr(request, "param", None)
     if not fixture_param:
