@@ -18,6 +18,8 @@
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Martin Barisits <martin.barisits@cern.ch>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 
 import random
 import string
@@ -40,16 +42,17 @@ from rucio.api.rule import add_replication_rule
 from rucio.api.scope import add_scope, list_scopes, get_scopes
 from rucio.api.subscription import add_subscription, list_subscriptions, list_subscription_rule_states, \
     get_subscription_by_id
-from rucio.common.config import config_get, config_get_bool
+from rucio.common.config import config_get_bool
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import api_update_return_dict, generate_uuid
 from rucio.core.rse import get_rse_id
 from rucio.core.vo import add_vo, vo_exists
 from rucio.daemons.abacus import rse as abacus_rse
 from rucio.daemons.judge import cleaner
-from rucio.daemons.reaper import reaper2
+from rucio.daemons.reaper import reaper
 from rucio.db.sqla import constants
 from rucio.tests.common import rse_name_generator
+from rucio.tests.common_server import get_vo
 
 
 @pytest.mark.noparallel(reason='uses pre-defined RSE, fails when run in parallel')
@@ -58,7 +61,7 @@ class TestApiExternalRepresentation(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
-            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+            cls.vo = {'vo': get_vo()}
             cls.new_vo = {'vo': 'new'}
             cls.multi_vo = True
             if not vo_exists(**cls.new_vo):
@@ -371,9 +374,9 @@ class TestApiExternalRepresentation(unittest.TestCase):
         # clean up files
         cleaner.run(once=True)
         if self.multi_vo:
-            reaper2.run(once=True, include_rses='vo=%s&(%s)' % (self.vo['vo'], rse_mock), greedy=True)
+            reaper.run(once=True, include_rses='vo=%s&(%s)' % (self.vo['vo'], rse_mock), greedy=True)
         else:
-            reaper2.run(once=True, include_rses=rse_mock, greedy=True)
+            reaper.run(once=True, include_rses=rse_mock, greedy=True)
         abacus_rse.run(once=True)
 
         out = api_rse.parse_rse_expression('%s|%s' % (self.rse_name, self.rse2_name), **self.vo)
@@ -416,7 +419,7 @@ class TestApiExternalRepresentation(unittest.TestCase):
                              lifetime=180, grouping='DATASET', account=new_acc_name, locked=False, subscription_id=sub_id,
                              source_replica_expression=self.rse4_name, activity='User Subscriptions', notify=None,
                              purge_replicas=False, ignore_availability=False, comment='', ask_approval=False, asynchronous=False,
-                             priority=0, split_container=False, meta='', issuer='root', **self.new_vo)
+                             delay_injection=None, priority=0, split_container=False, meta='', issuer='root', **self.new_vo)
 
         out = list_subscriptions(sub, **self.new_vo)
         out = list(out)
