@@ -21,7 +21,7 @@
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2014-2018
 # - Cheng-Hsi Chao <cheng-hsi.chao@cern.ch>, 2014
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2015
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2015-2021
 # - Frank Berghaus <frank.berghaus@cern.ch>, 2017-2018
 # - Tobias Wegner <twegner@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
@@ -30,12 +30,18 @@
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Tomas Javurek <tomas.javurek@cern.ch>, 2020
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
+# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 from __future__ import print_function
 
 import os
+import random
 import re
 import unittest
+from datetime import datetime, timedelta
 from os import remove, unlink, listdir, rmdir, stat, path, environ
 
 import pytest
@@ -49,23 +55,29 @@ from rucio.common.config import config_get, config_get_bool
 from rucio.common.types import InternalScope, InternalAccount
 from rucio.common.utils import generate_uuid, get_tmp_dir, md5, render_json
 from rucio.rse import rsemanager as rsemgr
-from rucio.tests.common import execute, account_name_generator, rse_name_generator, file_generator, scope_name_generator
+from rucio.tests.common import execute, account_name_generator, rse_name_generator, file_generator, scope_name_generator, get_long_vo
 
 
 class TestBinRucio(unittest.TestCase):
 
-    def setUp(self):
+    def conf_vo(self):
+        self.vo = {}
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
-            self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+            if environ.get('SUITE', 'all') != 'client':
+                # Server test, we can use short VO via DB for internal tests
+                from rucio.tests.common_server import get_vo
+                self.vo = {'vo': get_vo()}
+            else:
+                # Client-only test, only use config with no DB config
+                self.vo = {'vo': get_long_vo()}
             try:
                 remove(get_tmp_dir() + '/.rucio_root@%s/auth_token_root' % self.vo['vo'])
             except OSError as error:
                 if error.args[0] != 2:
                     raise error
 
-        else:
-            self.vo = {}
-
+    def setUp(self):
+        self.conf_vo()
         try:
             remove(get_tmp_dir() + '/.rucio_root/auth_token_root')
         except OSError as e:
@@ -261,6 +273,40 @@ class TestBinRucio(unittest.TestCase):
         assert exitcode != 0
         assert 'Distance from %s to %s already exists!' % (temprse2, temprse1) in err
 
+    def test_rse_delete_distance(self):
+        """CLIENT (ADMIN): Delete distance to RSE"""
+        # add RSEs
+        temprse1 = rse_name_generator()
+        cmd = 'rucio-admin rse add %s' % temprse1
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+        temprse2 = rse_name_generator()
+        cmd = 'rucio-admin rse add %s' % temprse2
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+
+        # add distance between the RSEs
+        cmd = 'rucio-admin rse add-distance --distance 1 --ranking 1 %s %s' % (temprse1, temprse2)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+
+        # delete distance OK
+        cmd = 'rucio-admin rse delete-distance %s %s' % (temprse1, temprse2)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+        assert "Deleted distance information from %s to %s." % (temprse1, temprse2) in out
+
+        # delete distance RSE not found
+        cmd = 'rucio-admin rse delete-distance %s %s' % (temprse1, generate_uuid())
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert 'RSE does not exist.' in err
+
     def test_upload(self):
         """CLIENT(USER): Upload"""
         tmp_val = rse_name_generator()
@@ -894,7 +940,7 @@ class TestBinRucio(unittest.TestCase):
         search = '{0} successfully downloaded'.format(tmp_file1[5:])  # triming '/tmp/' from filename
         assert re.search(search, err) is not None
 
-    def test_list_blacklisted_replicas(self):
+    def test_list_blocklisted_replicas(self):
         """CLIENT(USER): Rucio list replicas"""
         # add rse
         tmp_rse = rse_name_generator()
@@ -902,7 +948,7 @@ class TestBinRucio(unittest.TestCase):
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        cmd = 'rucio-admin rse add-protocol --hostname blacklistreplica --scheme file --prefix /rucio --port 0 --impl rucio.rse.protocols.posix.Default ' \
+        cmd = 'rucio-admin rse add-protocol --hostname blocklistreplica --scheme file --prefix /rucio --port 0 --impl rucio.rse.protocols.posix.Default ' \
               '--domain-json \'{"wan": {"read": 1, "write": 1, "delete": 1, "third_party_copy": 1}}\' %s' % tmp_rse
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
@@ -928,21 +974,21 @@ class TestBinRucio(unittest.TestCase):
         exitcode, out, err = execute(cmd)
         print(out, err)
 
-        # Listing the replica should work before blacklisting the RSE
+        # Listing the replica should work before blocklisting the RSE
         cmd = 'rucio list-file-replicas {}'.format(tmp_dataset)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
         assert tmp_rse in out
 
-        # Blacklist the rse
+        # Blocklist the rse
         cmd = 'rucio-admin rse update --rse {} --setting availability_read --value False'.format(tmp_rse)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
         assert not err
 
-        # list-file-replicas should, by default, list replicas from blacklisted rses
+        # list-file-replicas should, by default, list replicas from blocklisted rses
         cmd = 'rucio list-file-replicas {}'.format(tmp_dataset)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
@@ -1001,7 +1047,9 @@ class TestBinRucio(unittest.TestCase):
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out)
+        assert not err
         rule = out[:-1]  # triming new line character
+        assert re.match(r'^\w+$', rule)
         # check if rule exist for the file
         cmd = "rucio list-rules {0}:{1}".format(self.user, tmp_file1[5:])
         print(self.marker + cmd)
@@ -1009,6 +1057,52 @@ class TestBinRucio(unittest.TestCase):
         print(out, err)
         assert re.search(rule, out) is not None
 
+    def test_create_rule_delayed(self):
+        """CLIENT(USER): Rucio add rule delayed"""
+        tmp_file1 = file_generator()
+        # add files
+        cmd = 'rucio upload --rse {0} --scope {1} {2}'.format(self.def_rse, self.user, tmp_file1)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        # add rse
+        tmp_rse = rse_name_generator()
+        cmd = 'rucio-admin rse add {0}'.format(tmp_rse)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        # add quota
+        self.account_client.set_local_account_limit('root', tmp_rse, -1)
+        # add rse atributes
+        cmd = 'rucio-admin rse set-attribute --rse {0} --key spacetoken --value ATLASRULEDELAYED'.format(tmp_rse)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        # try adding rule with an incorrect delay-injection. Must fail
+        cmd = "rucio add-rule --delay-injection asdsaf {0}:{1} 1 'spacetoken=ATLASRULEDELAYED'".format(self.user, tmp_file1[5:])
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        assert err
+        # Add a correct rule
+        cmd = "rucio add-rule --delay-injection 3600 {0}:{1} 1 'spacetoken=ATLASRULEDELAYED'".format(self.user, tmp_file1[5:])
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert not err
+        rule = out[:-1]  # triming new line character
+        cmd = "rucio rule-info {0}".format(rule)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        out_lines = out.splitlines()
+        assert any(re.match(r'State:.* INJECT', line) for line in out_lines)
+        assert any(re.match(r'Locks OK/REPLICATING/STUCK:.* 0/0/0', line) for line in out_lines)
+        # Check that "Created at" is approximately 3600 seconds in the future
+        [created_at_line] = filter(lambda x: "Created at" in x, out_lines)
+        created_at = re.search(r'Created at:\s+(\d.*\d)$', created_at_line).group(1)
+        created_at = datetime.strptime(created_at, "%Y-%m-%d %H:%M:%S")
+        assert datetime.utcnow() + timedelta(seconds=3550) < created_at < datetime.utcnow() + timedelta(seconds=3650)
+
     def test_delete_rule(self):
         """CLIENT(USER): rule deletion"""
         self.account_client.set_local_account_limit('root', self.def_rse, -1)
@@ -1474,3 +1568,157 @@ class TestBinRucio(unittest.TestCase):
         assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out) is not None
         self.account_client.set_local_account_limit(account, rse, -1)
         self.account_client.set_global_account_limit(account, rse_exp, -1)
+
+    def test_get_set_delete_limits_rse(self):
+        """CLIENT(ADMIN): Get, set and delete RSE limits"""
+        name = generate_uuid()
+        value = random.randint(0, 100000)
+        name2 = generate_uuid()
+        value2 = random.randint(0, 100000)
+        name3 = generate_uuid()
+        value3 = account_name_generator()
+        cmd = 'rucio-admin rse set-limit %s %s %s' % (self.def_rse, name, value)
+        execute(cmd)
+        cmd = 'rucio-admin rse set-limit %s %s %s' % (self.def_rse, name2, value2)
+        execute(cmd)
+        cmd = 'rucio-admin rse info %s' % self.def_rse
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}: {1}".format(name, value), out) is not None
+        assert re.search("{0}: {1}".format(name2, value2), out) is not None
+        new_value = random.randint(100001, 999999999)
+        cmd = 'rucio-admin rse set-limit %s %s %s' % (self.def_rse, name, new_value)
+        execute(cmd)
+        cmd = 'rucio-admin rse info %s' % self.def_rse
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}: {1}".format(name, new_value), out) is not None
+        assert re.search("{0}: {1}".format(name, value), out) is None
+        assert re.search("{0}: {1}".format(name2, value2), out) is not None
+        cmd = 'rucio-admin rse delete-limit %s %s' % (self.def_rse, name)
+        execute(cmd)
+        cmd = 'rucio-admin rse info %s' % self.def_rse
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}: {1}".format(name, new_value), out) is None
+        assert re.search("{0}: {1}".format(name2, value2), out) is not None
+        cmd = 'rucio-admin rse delete-limit %s %s' % (self.def_rse, name)
+        exitcode, out, err = execute(cmd)
+        assert re.search('Limit {0} not defined in RSE {1}'.format(name, self.def_rse), err) is not None
+        cmd = 'rucio-admin rse set-limit %s %s %s' % (self.def_rse, name3, value3)
+        exitcode, out, err = execute(cmd)
+        assert re.search('The RSE limit value must be an integer', err) is not None
+        cmd = 'rucio-admin rse info %s' % self.def_rse
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}: {1}".format(name3, value3), out) is None
+        assert re.search("{0}: {1}".format(name2, value2), out) is not None
+
+    def test_upload_recursive_ok(self):
+        """CLIENT(USER): Upload and preserve folder structure"""
+        folder = 'folder_' + generate_uuid()
+        folder1 = '%s/folder_%s' % (folder, generate_uuid())
+        folder2 = '%s/folder_%s' % (folder, generate_uuid())
+        folder3 = '%s/folder_%s' % (folder, generate_uuid())
+        folder11 = '%s/folder_%s' % (folder1, generate_uuid())
+        folder12 = '%s/folder_%s' % (folder1, generate_uuid())
+        folder13 = '%s/folder_%s' % (folder1, generate_uuid())
+        file1 = 'file_%s' % generate_uuid()
+        file2 = 'file_%s' % generate_uuid()
+        cmd = 'mkdir %s' % folder
+        execute(cmd)
+        cmd = 'mkdir %s && mkdir %s && mkdir %s' % (folder1, folder2, folder3)
+        execute(cmd)
+        cmd = 'mkdir %s && mkdir %s && mkdir %s' % (folder11, folder12, folder13)
+        execute(cmd)
+        cmd = 'echo "%s" > %s/%s.txt' % (generate_uuid(), folder11, file1)
+        execute(cmd)
+        cmd = 'echo "%s" > %s/%s.txt' % (generate_uuid(), folder2, file2)
+        execute(cmd)
+        cmd = 'rucio upload --scope %s --rse %s --recursive %s/' % (self.user, self.def_rse, folder)
+        execute(cmd)
+        cmd = 'rucio list-content %s:%s' % (self.user, folder)
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, folder1.split('/')[-1]), out) is not None
+        assert re.search("{0}:{1}".format(self.user, folder2.split('/')[-1]), out) is not None
+        assert re.search("{0}:{1}".format(self.user, folder3.split('/')[-1]), out) is None
+        cmd = 'rucio list-content %s:%s' % (self.user, folder1.split('/')[-1])
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, folder11.split('/')[-1]), out) is not None
+        assert re.search("{0}:{1}".format(self.user, folder12.split('/')[-1]), out) is None
+        assert re.search("{0}:{1}".format(self.user, folder13.split('/')[-1]), out) is None
+        cmd = 'rucio list-content %s:%s' % (self.user, folder11.split('/')[-1])
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, file1), out) is not None
+        cmd = 'rucio list-content %s:%s' % (self.user, folder2.split('/')[-1])
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, file2), out) is not None
+        cmd = 'rm -rf %s' % folder
+        execute(cmd)
+
+    def test_upload_recursive_subfolder(self):
+        """CLIENT(USER): Upload and preserve folder structure in a subfolder"""
+        folder = 'folder_' + generate_uuid()
+        folder1 = '%s/folder_%s' % (folder, generate_uuid())
+        folder11 = '%s/folder_%s' % (folder1, generate_uuid())
+        file1 = 'file_%s' % generate_uuid()
+        cmd = 'mkdir %s' % (folder)
+        execute(cmd)
+        cmd = 'mkdir %s' % (folder1)
+        execute(cmd)
+        cmd = 'mkdir %s' % (folder11)
+        execute(cmd)
+        cmd = 'echo "%s" > %s/%s.txt' % (generate_uuid(), folder11, file1)
+        execute(cmd)
+        cmd = 'rucio upload --scope %s --rse %s --recursive %s/' % (self.user, self.def_rse, folder1)
+        execute(cmd)
+        cmd = 'rucio list-content %s:%s' % (self.user, folder)
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, folder1.split('/')[-1]), out) is None
+        cmd = 'rucio list-content %s:%s' % (self.user, folder1.split('/')[-1])
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, folder11.split('/')[-1]), out) is not None
+        cmd = 'rucio list-content %s:%s' % (self.user, folder11.split('/')[-1])
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, file1), out) is not None
+        cmd = 'rm -rf %s' % folder
+        execute(cmd)
+
+    def test_recursive_empty(self):
+        """CLIENT(USER): Upload and preserve folder structure with an empty folder"""
+        folder = 'folder_' + generate_uuid()
+        folder1 = '%s/folder_%s' % (folder, generate_uuid())
+        cmd = 'mkdir %s' % (folder)
+        execute(cmd)
+        cmd = 'mkdir %s' % (folder1)
+        execute(cmd)
+        cmd = 'rucio upload --scope %s --rse %s --recursive %s/' % (self.user, self.def_rse, folder)
+        execute(cmd)
+        cmd = 'rucio list-content %s:%s' % (self.user, folder)
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, folder1.split('/')[-1]), out) is None
+        cmd = 'rm -rf %s' % folder
+        execute(cmd)
+
+    def test_upload_recursive_only_files(self):
+        """CLIENT(USER): Upload and preserve folder structure only with files"""
+        folder = 'folder_' + generate_uuid()
+        file1 = 'file_%s' % generate_uuid()
+        file2 = 'file_%s' % generate_uuid()
+        file3 = 'file_%s' % generate_uuid()
+        cmd = 'mkdir %s' % folder
+        execute(cmd)
+        cmd = 'echo "%s" > %s/%s.txt' % (generate_uuid(), folder, file1)
+        execute(cmd)
+        cmd = 'echo "%s" > %s/%s.txt' % (generate_uuid(), folder, file2)
+        execute(cmd)
+        cmd = 'echo "%s" > %s/%s.txt' % (generate_uuid(), folder, file3)
+        execute(cmd)
+        cmd = 'rucio upload --scope %s --rse %s --recursive %s/' % (self.user, self.def_rse, folder)
+        execute(cmd)
+        cmd = 'rucio list-content %s:%s' % (self.user, folder)
+        exitcode, out, err = execute(cmd)
+        assert re.search("{0}:{1}".format(self.user, file1), out) is not None
+        assert re.search("{0}:{1}".format(self.user, file2), out) is not None
+        assert re.search("{0}:{1}".format(self.user, file3), out) is not None
+        cmd = 'rucio ls %s:%s' % (self.user, folder)
+        exitcode, out, err = execute(cmd)
+        assert re.search("DATASET", out) is not None
+        cmd = 'rm -rf %s' % folder
+        execute(cmd)
