@@ -14,7 +14,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2021
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2015
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2020
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
@@ -22,7 +22,10 @@
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
+# - Nick Smith <nick.smith@cern.ch>, 2021
 
+import itertools
 import unittest
 from hashlib import sha256
 
@@ -46,6 +49,7 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import DIDType, RuleState, ReplicaState
 from rucio.db.sqla.session import get_session
 from rucio.tests.common import rse_name_generator
+from rucio.tests.common_server import get_vo
 from rucio.tests.test_rule import create_files, tag_generator
 
 
@@ -56,7 +60,7 @@ class TestJudgeRepairer(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
-            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+            cls.vo = {'vo': get_vo()}
         else:
             cls.vo = {}
 
@@ -298,34 +302,50 @@ class TestJudgeRepairer(unittest.TestCase):
         assert(RuleState.REPLICATING == get_rule(rule_id)['state'])
         assert(get_replica_locks(scope=files[3]['scope'], name=files[3]['name'])[0].rse_id != old_rse_id)
 
-    def test_to_repair_a_rule_with_only_1_rse_whose_site_is_blacklisted(self):
-        """ JUDGE REPAIRER: Test to repair a rule with only 1 rse whose site is blacklisted"""
+    def test_to_repair_a_rule_with_only_1_rse_whose_site_is_blocklisted(self):
+        """ JUDGE REPAIRER: Test to repair a rule with only 1 rse whose site is blocklisted"""
 
         rse = rse_name_generator()
         rse_id = add_rse(rse, **self.vo)
-        update_rse(rse_id, {'availability_write': False})
         set_local_account_limit(self.jdoe, rse_id, -1)
-
         rule_repairer(once=True)  # Clean out the repairer
-        scope = InternalScope('mock', **self.vo)
-        files = create_files(4, scope, self.rse4_id, bytes=100)
-        dataset = 'dataset_' + str(uuid())
-        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
-        attach_dids(scope, dataset, files, self.jdoe)
 
-        rule_id = add_rule(dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=rse, grouping='DATASET', weight=None, lifetime=None, locked=False, subscription_id=None, ignore_availability=True, activity='DebugJudge')[0]
+        region = make_region().configure(
+            'dogpile.cache.memcached',
+            expiration_time=3600,
+            arguments={'url': config_get('cache', 'url', False, '127.0.0.1:11211'), 'distributed_lock': True}
+        )
 
-        assert(RuleState.STUCK == get_rule(rule_id)['state'])
-        rule_repairer(once=True)
+        def change_availability(new_value):
+            update_rse(rse_id, {'availability_write': new_value})
+            # clear cache
+            region.delete(sha256(rse.encode()).hexdigest())
 
-        # Stil assert STUCK because of ignore_availability:
-        assert(RuleState.STUCK == get_rule(rule_id)['state'])
+        for grouping, ignore_availability in itertools.product(["NONE", "DATASET", "ALL"], [True, False]):
+            scope = InternalScope('mock', **self.vo)
+            files = create_files(1, scope, self.rse4_id, bytes=100)
+            dataset = 'dataset_' + str(uuid())
+            add_did(scope, dataset, DIDType.DATASET, self.jdoe)
+            attach_dids(scope, dataset, files, self.jdoe)
 
-        region = make_region().configure('dogpile.cache.memcached',
-                                         expiration_time=3600,
-                                         arguments={'url': config_get('cache', 'url', False, '127.0.0.1:11211'), 'distributed_lock': True})
-        region.delete(sha256(rse.encode()).hexdigest())
+            if ignore_availability:
+                change_availability(False)
+                rule_id = add_rule(dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=rse, grouping=grouping, weight=None, lifetime=None, locked=False, subscription_id=None, ignore_availability=ignore_availability, activity='DebugJudge')[0]
+                assert(RuleState.STUCK == get_rule(rule_id)['state'])
 
-        update_rse(rse_id, {'availability_write': True})
-        rule_repairer(once=True)
-        assert(RuleState.REPLICATING == get_rule(rule_id)['state'])
+                rule_repairer(once=True)
+                assert(RuleState.REPLICATING == get_rule(rule_id)['state'])
+
+                change_availability(True)
+            else:
+                rule_id = add_rule(dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=rse, grouping=grouping, weight=None, lifetime=None, locked=False, subscription_id=None, ignore_availability=ignore_availability, activity='DebugJudge')[0]
+                failed_transfer(scope=scope, name=files[0]['name'], rse_id=get_replica_locks(scope=files[0]['scope'], name=files[0]['name'])[0].rse_id)
+                change_availability(False)
+                assert(RuleState.STUCK == get_rule(rule_id)['state'])
+
+                rule_repairer(once=True)
+                assert(RuleState.STUCK == get_rule(rule_id)['state'])
+
+                change_availability(True)
+                rule_repairer(once=True)
+                assert(RuleState.REPLICATING == get_rule(rule_id)['state'])
