@@ -17,9 +17,9 @@
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2018
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013-2014
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2018
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2021
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2015-2021
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
 # - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
@@ -28,27 +28,29 @@
 # - Luc Goossens <luc.goossens@cern.ch>, 2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Ilija Vukotic <ivukotic@uchicago.edu>, 2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Ilija Vukotic <ivukotic@uchicago.edu>, 2021
 # - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 from __future__ import print_function
+
 import hashlib
-import os
-import sys
 import time
 from datetime import datetime, timedelta
 from json import dumps, loads
+from unittest import mock
 from xml.etree import ElementTree
+
 import pytest
 import xmltodict
 from werkzeug.datastructures import MultiDict
+
 from rucio.client.ruleclient import RuleClient
-from rucio.core.config import set as cconfig_set
 from rucio.common.exception import (DataIdentifierNotFound, AccessDenied, UnsupportedOperation,
                                     RucioException, ReplicaIsLocked, ReplicaNotFound, ScopeNotFound,
                                     DatabaseException)
 from rucio.common.utils import generate_uuid, clean_surls, parse_response
+from rucio.core.config import set as cconfig_set
 from rucio.core.did import add_did, attach_dids, get_did, set_status, list_files, get_did_atime
 from rucio.core.replica import (add_replica, add_replicas, delete_replicas, get_replicas_state,
                                 get_replica, list_replicas, declare_bad_file_replicas, list_bad_replicas,
@@ -58,20 +60,15 @@ from rucio.core.rse import add_protocol, add_rse_attribute, del_rse_attribute
 from rucio.daemons.badreplicas.minos import run as minos_run
 from rucio.daemons.badreplicas.minos_temporary_expiration import run as minos_temp_run
 from rucio.daemons.badreplicas.necromancer import run as necromancer_run
+from rucio.db.sqla import models
 from rucio.db.sqla.constants import DIDType, ReplicaState, BadPFNStatus, OBSOLETE
+from rucio.db.sqla.session import transactional_session
 from rucio.rse import rsemanager as rsemgr
 from rucio.tests.common import execute, headers, auth, Mime, accept
 
 
-if sys.version_info >= (3, 3):
-    from unittest import mock
-else:
-    import mock
-
-# This method will be used by the mock to replace requests.get to VP server
-
-
 def mocked_VP_requests_get(*args, **kwargs):
+    """This method will be used by the mock to replace requests.get to VP server."""
     class MockResponse:
         def __init__(self, json_data, status_code):
             self.json_data = json_data
@@ -252,10 +249,11 @@ class TestReplicaCore:
         for file in files2:
             get_did(scope=file['scope'], name=file['name'])
 
-    def test_delete_replicas_from_datasets(self, rse_factory, mock_scope, root_account):
+    def test_delete_replicas_from_datasets_old(self, rse_factory, mock_scope, root_account):
         """ REPLICA (CORE): Delete replicas from dataset """
         _, rse_id = rse_factory.make_mock_rse()
 
+        cconfig_set(section='reaper', option='remove_open_did', value=False)
         tmp_dsn1 = 'dsn_%s' % generate_uuid()
         tmp_dsn2 = 'dsn_%s' % generate_uuid()
         nbfiles = 5
@@ -523,6 +521,33 @@ class TestReplicaCore:
         with pytest.raises(ReplicaNotFound):
             set_tombstone(rse_id, mock_scope, name)
 
+    def test_core_default_tombstone_correctly_set(self, rse_factory, did_factory, root_account):
+        """ REPLICA (CORE): Per-RSE default tombstone is correctly taken into consideration"""
+
+        # One RSE has an attribute set, the other uses the default value of "None" for tombstone
+        rse1, rse1_id = rse_factory.make_mock_rse()
+        rse2, rse2_id = rse_factory.make_mock_rse()
+        tombstone_delay = 3600
+        add_rse_attribute(rse_id=rse2_id, key='tombstone_delay', value=tombstone_delay)
+
+        # Will use the default tombstone delay
+        did1 = did_factory.random_did()
+        add_replica(rse1_id, bytes=4, account=root_account, **did1)
+        assert get_replica(rse1_id, **did1)['tombstone'] is None
+
+        # Will use the configured value on the RSE
+        did2 = did_factory.random_did()
+        add_replica(rse2_id, bytes=4, account=root_account, **did2)
+        tombstone = get_replica(rse2_id, **did2)['tombstone']
+        expected_tombstone = datetime.utcnow() + timedelta(seconds=tombstone_delay)
+        assert expected_tombstone - timedelta(minutes=5) < tombstone < expected_tombstone + timedelta(minutes=5)
+
+        # Adding rule removes the tombstone
+        RuleClient().add_replication_rule([{'name': did1['name'], 'scope': did1['scope'].external}], 1, rse1, locked=True)
+        assert get_replica(rse1_id, **did1)['tombstone'] is None
+        RuleClient().add_replication_rule([{'name': did2['name'], 'scope': did2['scope'].external}], 1, rse2, locked=True)
+        assert get_replica(rse2_id, **did2)['tombstone'] is None
+
     def test_list_replicas_with_updated_after(self, rse_factory, mock_scope, root_account):
         """ REPLICA (CORE): Add and list file replicas with updated_after filter """
         _, rse_id = rse_factory.make_mock_rse()
@@ -584,6 +609,32 @@ class TestReplicaCore:
         assert cov[rse3_id] == 500
 
 
+@pytest.mark.parametrize("core_config_mock", [{"table_content": [
+    ('reaper', 'remove_open_did', True)
+]}], indirect=True)
+@pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
+    'rucio.core.config', 'rucio.core.replica'
+]}], indirect=True)
+def test_delete_replicas_from_datasets_new(core_config_mock, caches_mock, rse_factory, mock_scope, root_account):
+    """ REPLICA (CORE): Delete replicas from dataset """
+    _, rse_id = rse_factory.make_mock_rse()
+
+    tmp_dsn1 = 'dsn_%s' % generate_uuid()
+    nbfiles = 5
+    files1 = [{'scope': mock_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+
+    add_did(scope=mock_scope, name=tmp_dsn1, type=DIDType.DATASET, account=root_account)
+
+    attach_dids(scope=mock_scope, name=tmp_dsn1, rse_id=rse_id, dids=files1, account=root_account)
+
+    set_status(scope=mock_scope, name=tmp_dsn1, open=False)
+
+    delete_replicas(rse_id=rse_id, files=files1)
+
+    with pytest.raises(DataIdentifierNotFound):
+        get_did(scope=mock_scope, name=tmp_dsn1)
+
+
 @pytest.mark.noparallel(reason='calls list_bad_replicas() and runs necromancer. Both act on all bad replicas without any filtering')
 def test_client_add_list_bad_replicas(rse_factory, replica_client, did_client):
     """ REPLICA (CLIENT): Add bad replicas"""
@@ -596,7 +647,7 @@ def test_client_add_list_bad_replicas(rse_factory, replica_client, did_client):
 
     # Listing replicas on deterministic RSE
     replicas, list_rep = [], []
-    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], all_states=True):
         replicas.extend(replica['rses'][rse1])
         list_rep.append(replica)
     r = replica_client.declare_bad_file_replicas(replicas, 'This is a good reason')
@@ -627,7 +678,7 @@ def test_client_add_list_bad_replicas(rse_factory, replica_client, did_client):
 
     # Listing replicas on non-deterministic RSE
     replicas, list_rep = [], []
-    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], all_states=True):
         replicas.extend(replica['rses'][rse2])
         list_rep.append(replica)
     print(replicas, list_rep)
@@ -662,7 +713,7 @@ def test_client_add_suspicious_replicas(rse_factory, replica_client):
     # Listing replicas on deterministic RSE
     replicas = []
     list_rep = []
-    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], all_states=True):
         replicas.extend(replica['rses'][rse1])
         list_rep.append(replica)
     r = replica_client.declare_suspicious_file_replicas(replicas, 'This is a good reason')
@@ -677,7 +728,7 @@ def test_client_add_suspicious_replicas(rse_factory, replica_client):
     # Listing replicas on non-deterministic RSE
     replicas = []
     list_rep = []
-    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], unavailable=True):
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm'], all_states=True):
         replicas.extend(replica['rses'][rse2])
         list_rep.append(replica)
     r = replica_client.declare_suspicious_file_replicas(replicas, 'This is a good reason')
@@ -829,7 +880,7 @@ def test_client_add_list_replicas(rse_factory, replica_client, mock_scope):
         file['state'] = 'A'
         files4.append(file)
     replica_client.update_replicas_states(rse2, files=files4)
-    replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files3], schemes=['file'], unavailable=True)]
+    replicas = [r for r in replica_client.list_replicas(dids=[{'scope': i['scope'], 'name': i['name']} for i in files3], schemes=['file'], all_states=True)]
     assert len(replicas) == 5
     for i in range(nbfiles):
         assert rse2 in replicas[i]['rses']
@@ -856,8 +907,36 @@ def test_client_access_denied_on_delete_replicas(rse_factory, mock_scope, replic
         assert len(replicas) == 1
 
 
-def test_client_list_blacklisted_replicas(rse_factory, did_factory, replica_client, did_client):
-    """ REPLICA (CLIENT): Blacklisted replicas are filtered in list replicas"""
+def test_client_list_replicas_on_did_without_replicas(rse_factory, did_factory, replica_client, did_client, root_account):
+    """ REPLICA (CLIENT): dids of type FILE, but without replicas, must be listed with empty pfns and rses"""
+    rse, _ = rse_factory.make_posix_rse()
+    file = did_factory.random_did()
+    dataset = did_factory.make_dataset()
+    container = did_factory.make_container()
+
+    @transactional_session
+    def __add_file_did_without_replica(session=None):
+        models.DataIdentifier(scope=file['scope'], name=file['name'], did_type=DIDType.FILE, bytes=1, adler32='0cc737eb', account=root_account).save(session=session, flush=False)
+
+    __add_file_did_without_replica()
+
+    # make all scopes external
+    file, dataset, container = ({'scope': did['scope'].external, 'name': did['name']} for did in (file, dataset, container))
+
+    did_client.add_files_to_dataset(files=[file], **dataset)
+    did_client.add_datasets_to_container(dsns=[dataset], **container)
+
+    replicas = list(replica_client.list_replicas(dids=[file]))
+    assert len(replicas) == 1
+    assert not replicas[0]['rses']
+    assert not replicas[0]['pfns']
+    # TODO: fix listing dids without replicas from datasets and containers and uncomment the following 2 asserts
+    # assert list(replica_client.list_replicas(dids=[dataset]))
+    # assert list(replica_client.list_replicas(dids=[container]))
+
+
+def test_client_list_blocklisted_replicas(rse_factory, did_factory, replica_client, did_client):
+    """ REPLICA (CLIENT): Blocklisted replicas are filtered in list replicas"""
 
     rse, _ = rse_factory.make_posix_rse()
     file = did_factory.upload_test_file(rse)
@@ -904,7 +983,7 @@ def test_client_add_temporary_unavailable_pfns(rse_factory, mock_scope, replica_
 
     # Listing replicas on deterministic RSE
     list_rep = []
-    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['file'], unavailable=True):
+    for replica in replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['file'], all_states=True):
         pfn = list(replica['pfns'].keys())[0]
         list_rep.append(pfn)
 
@@ -1019,7 +1098,7 @@ class TestReplicaMetalink:
 
         ml = xmltodict.parse(replica_client.list_replicas(files,
                                                           metalink=4,
-                                                          unavailable=True,
+                                                          all_states=True,
                                                           schemes=['https', 'sftp', 'file']),
                              xml_attribs=False)
         assert 3 == len(ml['metalink']['file']['url'])
@@ -1099,88 +1178,52 @@ def test_client_list_replicas_streaming_error(content_type, vo, did_client, repl
         raise DatabaseException('Database error for testing')
 
     json_data = dumps({'dids': [{'scope': 'mock', 'name': generate_uuid()}]})
-    rest_backend = os.environ.get('REST_BACKEND', 'webpy')
-    if rest_backend == 'webpy':
-        def list_replicas_on_api():
-            class MockedHTTPError(Exception):
-                def __init__(self, status_code, exc_cls, exc_msg):
-                    super(MockedHTTPError, self).__init__("MockedHTTPError %s, %s: %s" % (status_code, exc_cls, exc_msg))
 
-                @classmethod
-                def generate(cls, *args, **kwargs):
-                    raise cls(*args, **kwargs)
+    def list_replicas_on_api():
+        from werkzeug.datastructures import Headers
 
-            class FakeCtx:
-                env = {
-                    'issuer': 'root',
-                    'vo': vo,
-                    'request_id': generate_uuid(),
-                    'HTTP_ACCEPT': content_type,
-                }
-                query = None
-                ip = '127.0.0.1'
+        class FakeRequest:
+            class FakeAcceptMimetypes:
+                provided = False
+                best_match = mock.MagicMock(return_value=content_type)
 
-            with mock.patch('rucio.web.rest.common.ctx', new=FakeCtx()), \
-                    mock.patch('rucio.web.rest.replica.ctx', new=FakeCtx()), \
-                    mock.patch('rucio.web.rest.replica.data', return_value=json_data), \
-                    mock.patch('rucio.web.rest.replica.header'), \
-                    mock.patch('rucio.web.rest.replica.generate_http_error', side_effect=MockedHTTPError.generate), \
-                    mock.patch('rucio.web.rest.replica.list_replicas', side_effect=api_returns):
-                from rucio.web.rest.replica import ListReplicas
-                list_replicas_restapi = ListReplicas()
-                with pytest.raises(MockedHTTPError, match='MockedHTTPError 500, DatabaseException: Database error for testing'):
-                    for element in list_replicas_restapi.POST():
-                        yield element
+            environ = {
+                'issuer': 'root',
+                'vo': vo,
+                'request_id': generate_uuid(),
+            }
+            query_string = None
+            args = MultiDict()
+            data = json_data
+            get_data = mock.MagicMock(return_value=json_data)
+            headers = Headers()
+            accept_mimetypes = FakeAcceptMimetypes()
+            remote_addr = '127.0.0.1'
 
-    elif rest_backend == 'flask':
-        def list_replicas_on_api():
-            from werkzeug.datastructures import Headers
+        response_mock = mock.Mock(return_value=None)
 
-            class FakeRequest:
-                class FakeAcceptMimetypes:
-                    provided = False
-                    best_match = mock.MagicMock(return_value=content_type)
+        class FakeFlask:
+            request = FakeRequest()
+            abort = mock.MagicMock()
+            Response = response_mock
 
-                environ = {
-                    'issuer': 'root',
-                    'vo': vo,
-                    'request_id': generate_uuid(),
-                }
-                query_string = None
-                args = MultiDict()
-                data = json_data
-                get_data = mock.MagicMock(return_value=json_data)
-                headers = Headers()
-                accept_mimetypes = FakeAcceptMimetypes()
-                remote_addr = '127.0.0.1'
-
-            response_mock = mock.Mock(return_value=None)
-
-            class FakeFlask:
-                request = FakeRequest()
-                abort = mock.MagicMock()
-                Response = response_mock
-
-            with mock.patch('rucio.web.rest.flaskapi.v1.common.flask', new=FakeFlask()), \
-                    mock.patch('rucio.web.rest.flaskapi.v1.replicas.request', new=FakeRequest()), \
-                    mock.patch('rucio.web.rest.flaskapi.v1.replicas.list_replicas', side_effect=api_returns):
-                from rucio.web.rest.flaskapi.v1.replicas import ListReplicas
-                list_replicas_restapi = ListReplicas()
-                list_replicas_restapi.post()
-                # for debugging when this test fails
-                print(f'Response({response_mock.call_args})')
-                print(f'  args = {response_mock.call_args[0]}')
-                print(f'kwargs = {response_mock.call_args[1]}')
-                assert response_mock.call_args[1]['content_type'] == content_type
-                response_iter = response_mock.call_args[0][0]
-                assert response_iter != '', 'unexpected empty response'
-                # since we're directly accessing the generator for Flask, there is no error handling
-                with pytest.raises(DatabaseException, match='Database error for testing'):
-                    for element in response_iter:
-                        yield element
-
-    else:
-        return pytest.xfail('unknown REST_BACKEND: ' + rest_backend)
+        with mock.patch('rucio.web.rest.flaskapi.v1.common.flask', new=FakeFlask()), \
+                mock.patch('rucio.web.rest.flaskapi.v1.replicas.request', new=FakeRequest()), \
+                mock.patch('rucio.web.rest.flaskapi.v1.replicas.list_replicas', side_effect=api_returns):
+            from rucio.web.rest.flaskapi.v1.replicas import ListReplicas
+            list_replicas_restapi = ListReplicas()
+            list_replicas_restapi.post()
+            # for debugging when this test fails
+            print(f'Response({response_mock.call_args})')
+            print(f'  args = {response_mock.call_args[0]}')
+            print(f'kwargs = {response_mock.call_args[1]}')
+            assert response_mock.call_args[1]['content_type'] == content_type
+            response_iter = response_mock.call_args[0][0]
+            assert response_iter != '', 'unexpected empty response'
+            # since we're directly accessing the generator for Flask, there is no error handling
+            with pytest.raises(DatabaseException, match='Database error for testing'):
+                for element in response_iter:
+                    yield element
 
     if content_type == Mime.METALINK:
         # for metalink, this builds the incomplete XML that should be returned by the API on error
